@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -165,7 +160,7 @@ class CControllerPopupGeneric extends CController {
 			],
 			'templates' => [
 				'title' => _('Templates'),
-				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
+				'min_user_type' => USER_TYPE_ZABBIX_USER,
 				'allowed_src_fields' => 'hostid,host',
 				'form' => [
 					'name' => 'templateform',
@@ -214,7 +209,7 @@ class CControllerPopupGeneric extends CController {
 			'proxies' => [
 				'title' => _('Proxies'),
 				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
-				'allowed_src_fields' => 'proxyid,host',
+				'allowed_src_fields' => 'proxyid,name',
 				'form' => [
 					'name' => 'proxiesform',
 					'id' => 'proxies'
@@ -223,6 +218,18 @@ class CControllerPopupGeneric extends CController {
 					_('Name')
 				]
 			],
+			'proxy_groups' => [
+				'title' => _('Proxy groups'),
+				'min_user_type' => USER_TYPE_ZABBIX_ADMIN,
+				'allowed_src_fields' => 'proxy_groupid,name',
+				'form' => [
+					'name' => 'proxy_groups_form',
+					'id' => 'proxy-groups'
+				],
+				'table_columns' => [
+					_('Name')
+				]
+			],
 			'triggers' => [
 				'title' => _('Triggers'),
 				'min_user_type' => USER_TYPE_ZABBIX_USER,
@@ -300,8 +307,8 @@ class CControllerPopupGeneric extends CController {
 					'id' => 'items'
 				],
 				'table_columns' => [
-					_('Name'),
-					_('Key'),
+					(new CColHeader(_('Name')))->addStyle('width: 30%;'),
+					(new CColHeader(_('Key')))->addStyle('width: 30%;'),
 					_('Type'),
 					_('Type of information'),
 					_('Status')
@@ -316,8 +323,8 @@ class CControllerPopupGeneric extends CController {
 					'id' => 'items'
 				],
 				'table_columns' => [
-					_('Name'),
-					_('Key'),
+					(new CColHeader(_('Name')))->addStyle('width: 30%;'),
+					(new CColHeader(_('Key')))->addStyle('width: 30%;'),
 					_('Type'),
 					_('Type of information'),
 					_('Status')
@@ -329,7 +336,8 @@ class CControllerPopupGeneric extends CController {
 				'allowed_src_fields' => 'key',
 				'table_columns' => [
 					_('Key'),
-					_('Name')
+					_('Name'),
+					''
 				]
 			],
 			'graphs' => [
@@ -367,8 +375,8 @@ class CControllerPopupGeneric extends CController {
 					'id' => 'items'
 				],
 				'table_columns' => [
-					_('Name'),
-					_('Key'),
+					(new CColHeader(_('Name')))->addStyle('width: 30%;'),
+					(new CColHeader(_('Key')))->addStyle('width: 30%;'),
 					_('Type'),
 					_('Type of information')
 				]
@@ -554,6 +562,8 @@ class CControllerPopupGeneric extends CController {
 			'real_hosts' =>							'in 1',
 			'normal_only' =>						'in 1',
 			'with_graphs' =>						'in 1',
+			'with_hosts' =>							'in 1',
+			'with_templates' =>						'in 1',
 			'with_graph_prototypes' =>				'in 1',
 			'with_items' =>							'in 1',
 			'with_simple_graph_items' =>			'in 1',
@@ -581,6 +591,8 @@ class CControllerPopupGeneric extends CController {
 			'host_pattern' =>						'array|not_empty',
 			'host_pattern_wildcard_allowed' =>		'in 1',
 			'host_pattern_multiple' =>				'in 1',
+			'hide_host_filter' =>					'in 1',
+			'resolve_macros' =>						'in 1',
 			'exclude_provisioned' =>				'in 1'
 		];
 
@@ -602,15 +614,6 @@ class CControllerPopupGeneric extends CController {
 			$this->disableids = $this->getInput('disableids');
 		}
 
-		if ($ret && $this->getInput('value_types', [])) {
-			foreach ($this->getInput('value_types') as $value_type) {
-				if (!is_numeric($value_type) || $value_type < 0 || $value_type > 15) {
-					error(_s('Incorrect value "%1$s" for "%2$s" field.', $value_type, 'value_types'));
-					$ret = false;
-				}
-			}
-		}
-
 		if (!$ret) {
 			$this->setResponse(
 				(new CControllerResponseData(['main_block' => json_encode([
@@ -877,7 +880,8 @@ class CControllerPopupGeneric extends CController {
 
 		// Host dropdown.
 		if (in_array($this->source_table, self::POPUPS_HAVING_HOST_FILTER)
-				&& ($this->source_table !== 'item_prototypes' || !$this->page_options['parent_discoveryid'])) {
+				&& ($this->source_table !== 'item_prototypes' || !$this->page_options['parent_discoveryid'])
+				&& !$this->hasInput('hide_host_filter')) {
 
 			$src_name = 'hosts';
 			if (!array_key_exists('monitored_hosts', $host_options)
@@ -1254,7 +1258,7 @@ class CControllerPopupGeneric extends CController {
 
 			case 'users':
 				$options += [
-					'output' => ['userid', 'username', 'name', 'surname', 'type', 'theme', 'lang']
+					'output' => ['userid', 'username', 'name', 'surname']
 				];
 
 				if ($this->hasInput('exclude_provisioned')) {
@@ -1343,6 +1347,10 @@ class CControllerPopupGeneric extends CController {
 					$options['with_httptests'] = true;
 				}
 
+				if ($this->hasInput('with_hosts')) {
+					$options['with_hosts'] = true;
+				}
+
 				if ($this->hasInput('with_items')) {
 					$options['with_items'] = true;
 				}
@@ -1382,6 +1390,10 @@ class CControllerPopupGeneric extends CController {
 					$options['with_items'] = true;
 				}
 
+				if ($this->hasInput('with_templates')) {
+					$options['with_templates'] = true;
+				}
+
 				$records = API::TemplateGroup()->get($options);
 				if ($this->hasInput('enrich_parent_groups')) {
 					$records = enrichParentTemplateGroups($records);
@@ -1475,8 +1487,12 @@ class CControllerPopupGeneric extends CController {
 
 			case 'items':
 			case 'item_prototypes':
+				$name_field = $this->source_table === 'items' && $this->getInput('resolve_macros', 0)
+					? 'name_resolved'
+					: 'name';
+
 				$options += [
-					'output' => ['itemid', 'name', 'key_', 'flags', 'type', 'value_type', 'status'],
+					'output' => ['itemid', $name_field, 'key_', 'flags', 'type', 'value_type', 'status'],
 					'selectHosts' => ['name'],
 					'templated' => $this->hasInput('templated_hosts') ? true : null
 				];
@@ -1508,13 +1524,19 @@ class CControllerPopupGeneric extends CController {
 					$records = API::ItemPrototype()->get($options);
 				}
 				else {
-					if ($this->hasInput('normal_only')) {
-						$options['filter']['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
-					}
+					$records = [];
 
-					$records = !$this->host_preselect_required || $this->hostids
-						? API::Item()->get($options + ['webitems' => true])
-						: [];
+					if (!$this->host_preselect_required || $this->hostids) {
+						if ($this->hasInput('normal_only')) {
+							$options['filter']['flags'] = ZBX_FLAG_DISCOVERY_NORMAL;
+						}
+
+						$records = API::Item()->get($options + ['webitems' => true]);
+
+						if ($this->getInput('resolve_macros', 0)) {
+							$records = CArrayHelper::renameObjectsKeys($records, ['name_resolved' => 'name']);
+						}
+					}
 				}
 
 				CArrayHelper::sort($records, ['name']);
@@ -1594,7 +1616,7 @@ class CControllerPopupGeneric extends CController {
 
 			case 'dchecks':
 				$records = API::DRule()->get([
-					'selectDChecks' => ['dcheckid', 'type', 'key_', 'ports'],
+					'selectDChecks' => ['dcheckid', 'type', 'key_', 'ports', 'allow_redirect'],
 					'output' => ['druleid', 'name']
 				]);
 
@@ -1603,12 +1625,22 @@ class CControllerPopupGeneric extends CController {
 
 			case 'proxies':
 				$options += [
-					'output' => ['proxyid', 'host']
+					'output' => ['proxyid', 'name']
 				];
 
 				$records = API::Proxy()->get($options);
-				CArrayHelper::sort($records, ['host']);
-				$records = CArrayHelper::renameObjectsKeys($records, ['proxyid' => 'id', 'host' => 'name']);
+				CArrayHelper::sort($records, ['name']);
+				$records = CArrayHelper::renameObjectsKeys($records, ['proxyid' => 'id']);
+				break;
+
+			case 'proxy_groups':
+				$options += [
+					'output' => ['proxy_groupid', 'name']
+				];
+
+				$records = API::ProxyGroup()->get($options);
+				CArrayHelper::sort($records, ['name']);
+				$records = CArrayHelper::renameObjectsKeys($records, ['proxy_groupid' => 'id']);
 				break;
 
 			case 'roles':
