@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -27,12 +22,19 @@
 $this->addJsFile('class.dashboard.js');
 $this->addJsFile('class.dashboard.page.js');
 $this->addJsFile('class.dashboard.widget.placeholder.js');
-$this->addJsFile('class.sortable.js');
+$this->addJsFile('class.form.fieldset.collapsible.js');
 $this->addJsFile('class.tagfilteritem.js');
+$this->addJsFile('class.widgets-data.js');
+$this->addJsFile('class.widget-base.js');
 $this->addJsFile('class.widget.js');
 $this->addJsFile('class.widget.inaccessible.js');
 $this->addJsFile('class.widget.iterator.js');
+$this->addJsFile('class.widget.misconfigured.js');
 $this->addJsFile('class.widget.paste-placeholder.js');
+$this->addJsFile('class.widget-field.checkbox-list.js');
+$this->addJsFile('class.widget-field.multiselect.js');
+$this->addJsFile('class.widget-field.time-period.js');
+$this->addJsFile('class.widget-select.popup.js');
 
 if (array_key_exists('error', $data)) {
 	show_error_message($data['error']);
@@ -40,6 +42,7 @@ if (array_key_exists('error', $data)) {
 	return;
 }
 
+$this->addJsFile('d3.js');
 $this->addJsFile('class.calendar.js');
 $this->addJsFile('class.cnavtree.js');
 $this->addJsFile('class.coverride.js');
@@ -50,10 +53,12 @@ $this->addJsFile('class.svg.map.js');
 $this->addJsFile('colorpicker.js');
 $this->addJsFile('flickerfreescreen.js');
 $this->addJsFile('gtlc.js');
+$this->addJsFile('items.js');
 $this->addJsFile('layout.mode.js');
 $this->addJsFile('leaflet.js');
 $this->addJsFile('leaflet.markercluster.js');
 $this->addJsFile('class.geomaps.js');
+$this->addJsFile('multilineinput.js');
 
 $this->includeJsFile('monitoring.dashboard.view.js.php');
 
@@ -64,24 +69,25 @@ $web_layout_mode = $this->getLayoutMode();
 
 $main_filter_form = null;
 
-if ($data['dynamic']['has_dynamic_widgets']) {
+if (array_key_exists(CWidgetsData::DATA_TYPE_HOST_ID, $data['broadcast_requirements'])
+		|| array_key_exists(CWidgetsData::DATA_TYPE_HOST_IDS, $data['broadcast_requirements'])) {
 	$main_filter_form = (new CForm('get'))
 		->setAttribute('name', 'dashboard_filter')
 		->setAttribute('aria-label', _('Main filter'))
 		->addVar('action', 'dashboard.view')
 		->addItem([
-			(new CLabel(_('Host'), 'dynamic_hostid_ms'))->addStyle('margin-right: 5px;'),
+			(new CLabel(_('Host'), 'dashboard_hostid_ms'))->addStyle('margin-right: 5px;'),
 			(new CMultiSelect([
-				'name' => 'dynamic_hostid',
+				'name' => 'dashboard_hostid',
 				'object_name' => 'hosts',
-				'data' => $data['dynamic']['host'] ? [$data['dynamic']['host']] : [],
+				'data' => $data['dashboard_host'] ? [$data['dashboard_host']] : [],
 				'multiple' => false,
 				'popup' => [
 					'parameters' => [
 						'srctbl' => 'hosts',
 						'srcfld1' => 'hostid',
 						'dstfrm' => 'dashboard_filter',
-						'dstfld1' => 'dynamic_hostid',
+						'dstfld1' => 'dashboard_hostid',
 						'monitored_hosts' => true,
 						'with_items' => true
 					]
@@ -106,36 +112,37 @@ $html_page = (new CHtmlPage())
 							->setAttribute('aria-disabled', !$data['dashboard']['editable'] ? 'true' : null)
 					)
 					->addItem(
-						(new CButton(null, NBSP()))
+						(new CSimpleButton())
 							->addClass(ZBX_STYLE_BTN_ACTION)
+							->addClass(ZBX_ICON_MENU)
 							->setId('dashboard-actions')
 							->setTitle(_('Actions'))
-							->setEnabled($data['dashboard']['can_edit_dashboards']
-								|| $data['dashboard']['can_view_reports']
-							)
+							->setEnabled($data['dashboard']['can_edit_dashboards'] || $data['can_view_reports'])
 							->setAttribute('aria-haspopup', true)
 							->setMenuPopup(CMenuPopupHelper::getDashboard($data['dashboard']['dashboardid'],
-								$data['dashboard']['editable'], $data['dashboard']['has_related_reports'],
-								$data['dashboard']['can_edit_dashboards'], $data['dashboard']['can_view_reports'],
-								$data['dashboard']['can_create_reports']
+								$data['dashboard']['editable'], $data['has_related_reports'],
+								$data['dashboard']['can_edit_dashboards'], $data['can_view_reports'],
+								$data['can_create_reports']
 							))
 					)
 					->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]))
 			))->setAttribute('aria-label', _('Content controls')))
 			->addItem((new CListItem(
 				(new CTag('nav', true, new CList([
-					(new CButton('dashboard-config'))->addClass(ZBX_STYLE_BTN_DASHBOARD_CONF),
+					(new CButton('dashboard-config'))
+						->addClass(ZBX_STYLE_BTN_ICON)
+						->addClass(ZBX_ICON_COG_FILLED),
 					(new CList())
 						->addClass(ZBX_STYLE_BTN_SPLIT)
 						->addItem(
-							(new CButton('dashboard-add-widget',
-								[(new CSpan())->addClass(ZBX_STYLE_PLUS_ICON), _('Add')]
-							))->addClass(ZBX_STYLE_BTN_ALT)
+							(new CButton('dashboard-add-widget', _('Add')))
+								->addClass(ZBX_STYLE_BTN_ALT)
+								->addClass(ZBX_ICON_PLUS_SMALL)
 						)
 						->addItem(
-							(new CButton('dashboard-add', ZWSPACE()))
+							(new CButton('dashboard-add'))
 								->addClass(ZBX_STYLE_BTN_ALT)
-								->addClass(ZBX_STYLE_BTN_TOGGLE_CHEVRON)
+								->addClass(ZBX_ICON_CHEVRON_DOWN_SMALL)
 						),
 					(new CButton('dashboard-save', _('Save changes'))),
 					(new CLink(_('Cancel'), '#'))->setId('dashboard-cancel'),
@@ -146,16 +153,18 @@ $html_page = (new CHtmlPage())
 			))->addStyle('display: none'))
 	)
 	->setKioskModeControls(
-		(count($data['dashboard']['pages']) > 1)
+		count($data['dashboard']['pages']) > 1
 			? (new CList())
 				->addClass(ZBX_STYLE_DASHBOARD_KIOSKMODE_CONTROLS)
 				->addItem(
-					(new CSimpleButton(null))
+					(new CSimpleButton())
+						->addClass(ZBX_ICON_CHEVRON_LEFT)
 						->addClass(ZBX_STYLE_BTN_DASHBOARD_KIOSKMODE_PREVIOUS_PAGE)
 						->setTitle(_('Previous page'))
 				)
 				->addItem(
-					(new CSimpleButton(null))
+					(new CSimpleButton())
+						->addClass(ZBX_ICON_PAUSE)
 						->addClass(ZBX_STYLE_BTN_DASHBOARD_KIOSKMODE_TOGGLE_SLIDESHOW)
 						->setTitle(($data['dashboard']['dashboardid'] !== null && $data['dashboard']['auto_start'] == 1)
 							? _s('Stop slideshow')
@@ -168,7 +177,8 @@ $html_page = (new CHtmlPage())
 						)
 				)
 				->addItem(
-					(new CSimpleButton(null))
+					(new CSimpleButton())
+						->addClass(ZBX_ICON_CHEVRON_RIGHT)
 						->addClass(ZBX_STYLE_BTN_DASHBOARD_KIOSKMODE_NEXT_PAGE)
 						->setTitle(_('Next page'))
 				)
@@ -188,14 +198,15 @@ $html_page = (new CHtmlPage())
 			->addClass(ZBX_STYLE_SELECTED)
 	])));
 
-if ($data['has_time_selector']) {
+if (array_key_exists(CWidgetsData::DATA_TYPE_TIME_PERIOD, $data['broadcast_requirements'])) {
 	$html_page->addItem(
 		(new CFilter())
-			->setProfile($data['time_period']['profileIdx'], $data['time_period']['profileIdx2'])
+			->setProfile($data['dashboard_time_period']['profileIdx'], $data['dashboard_time_period']['profileIdx2'])
 			->setActiveTab($data['active_tab'])
-			->addTimeSelector($data['time_period']['from'], $data['time_period']['to'],
+			->addTimeSelector($data['dashboard_time_period']['from'], $data['dashboard_time_period']['to'],
 				$web_layout_mode != ZBX_LAYOUT_KIOSKMODE
 			)
+			->preventHistoryUpdates()
 	);
 }
 
@@ -218,20 +229,18 @@ if ($web_layout_mode != ZBX_LAYOUT_KIOSKMODE) {
 				(new CDiv())
 					->addClass(ZBX_STYLE_DASHBOARD_NAVIGATION_CONTROLS)
 					->addItem([
-						(new CSimpleButton())
-							->addClass(ZBX_STYLE_DASHBOARD_PREVIOUS_PAGE)
-							->addClass('btn-iterator-page-previous')
+						(new CButtonIcon(ZBX_ICON_CHEVRON_LEFT, _('Previous page')))
+							->addClass(ZBX_STYLE_BTN_DASHBOARD_PREVIOUS_PAGE)
 							->setEnabled(false),
-						(new CSimpleButton())
-							->addClass(ZBX_STYLE_DASHBOARD_NEXT_PAGE)
-							->addClass('btn-iterator-page-next')
+						(new CButtonIcon(ZBX_ICON_CHEVRON_RIGHT, _('Next page')))
+							->addClass(ZBX_STYLE_BTN_DASHBOARD_NEXT_PAGE)
 							->setEnabled(false),
 						(new CSimpleButton([
 							(new CSpan(_s('Start slideshow')))->addClass('slideshow-state-stopped'),
 							(new CSpan(_s('Stop slideshow')))->addClass('slideshow-state-started')
 						]))
+							->addClass(ZBX_STYLE_BTN_DASHBOARD_TOGGLE_SLIDESHOW)
 							->addClass(ZBX_STYLE_BTN_ALT)
-							->addClass(ZBX_STYLE_DASHBOARD_TOGGLE_SLIDESHOW)
 							->addClass(
 								($data['dashboard']['dashboardid'] !== null && $data['dashboard']['auto_start'] == 1)
 									? 'slideshow-state-started'
@@ -254,9 +263,9 @@ $html_page
 		'widget_defaults' => $data['widget_defaults'],
 		'widget_last_type' => $data['widget_last_type'],
 		'configuration_hash' => $data['configuration_hash'],
-		'has_time_selector' => $data['has_time_selector'],
-		'time_period' => $data['time_period'],
-		'dynamic' => $data['dynamic'],
+		'broadcast_requirements' => $data['broadcast_requirements'],
+		'dashboard_host' => $data['dashboard_host'],
+		'dashboard_time_period' => $data['dashboard_time_period'],
 		'web_layout_mode' => $web_layout_mode,
 		'clone' => $data['clone']
 	]).');
