@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -70,7 +65,6 @@ if (array_key_exists('hosts', $data['filter'])) {
 	$multiselect_options['popup']['parameters']['dstfrm'] = $header_form->getId();
 
 	$host_ms = (new CMultiSelect($multiselect_options))->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH);
-
 	if ($multiselect_options['readonly']) {
 		$host_ms->setTitle(_('You cannot switch hosts for current selection.'));
 	}
@@ -95,6 +89,39 @@ if (array_key_exists('templates', $data['filter'])) {
 
 // Show Type dropdown in header for help items.
 if ($data['popup_type'] === 'help_items') {
+	switch ($options['itemtype']) {
+		case ITEM_TYPE_ZABBIX:
+		case ITEM_TYPE_ZABBIX_ACTIVE:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_ZABBIX_AGENT;
+			break;
+
+		case ITEM_TYPE_SIMPLE:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_SIMPLE_CHECK;
+			break;
+
+		case ITEM_TYPE_SNMPTRAP:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_SNMP_TRAP;
+			break;
+
+		case ITEM_TYPE_INTERNAL:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_ZABBIX_INTERNAL;
+			break;
+
+		case ITEM_TYPE_DB_MONITOR:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_DB_MONITOR;
+			break;
+
+		case ITEM_TYPE_IPMI:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_IPMI_AGENT;
+			break;
+
+		case ITEM_TYPE_JMX:
+			$popup_doc_url = CDocHelper::ITEM_TYPES_JMX_AGENT;
+			break;
+	}
+
+	$output['doc_url'] = CDocHelper::getUrl($popup_doc_url);
+
 	$types_select = (new CSelect('itemtype'))
 		->setId('itemtype')
 		->setFocusableElementId('label-itemtype')
@@ -145,7 +172,7 @@ if ($data['multiselect'] && $form !== null) {
 $table = (new CTableInfo())->setHeader(array_merge($table_columns, $data['table_columns']));
 
 if ($data['preselect_required']) {
-	$table->setNoDataMessage(_('Specify some filter condition to see the values.'));
+	$table->setNoDataMessage(_('Filter is not set'), _('Use the filter to display results'), ZBX_ICON_FILTER_LARGE);
 }
 
 // Output table rows.
@@ -154,6 +181,7 @@ switch ($data['popup_type']) {
 	case 'template_groups':
 	case 'host_groups':
 	case 'proxies':
+	case 'proxy_groups':
 	case 'host_templates':
 	case 'templates':
 	case 'drules':
@@ -361,6 +389,10 @@ switch ($data['popup_type']) {
 					'status' => $trigger['status'],
 					'host' => $trigger['hostname']
 				];
+
+				if ($data['popup_type'] === 'trigger_prototypes') {
+					$trigger['prototype'] = '1';
+				}
 			}
 		}
 		unset($trigger);
@@ -391,7 +423,7 @@ switch ($data['popup_type']) {
 						.getElementById(this.dataset.dstfld1)
 						.dispatchEvent(new CustomEvent("help_items.paste"));
 
-					updateItemFormElements();
+					window.updateItemFormElements && updateItemFormElements();
 
 					if (this.dataset.dstfld2 in values) {
 						popup_generic.setPopupOpenerFieldValues({[this.dataset.dstfld2]: values[this.dataset.dstfld2]});
@@ -399,7 +431,14 @@ switch ($data['popup_type']) {
 
 					popup_generic.closePopup(event);
 				');
-			$table->addRow([$name, $item['description']]);
+
+			$documentation_link = (new CLink(null, CDocHelper::getUrl($item['documentation_link'])))
+				->addClass(ZBX_STYLE_BTN_ICON)
+				->addClass(ZBX_ICON_HELP)
+				->setTitle(_('Help'))
+				->setTarget('_blank');
+
+			$table->addRow([$name, $item['description'], $documentation_link]);
 		}
 		unset($data['table_records']);
 		break;
@@ -408,7 +447,9 @@ switch ($data['popup_type']) {
 		foreach ($data['table_records'] as $d_rule) {
 			foreach ($d_rule['dchecks'] as $d_check) {
 				$name = $d_rule['name'].
-					NAME_DELIMITER.discovery_check2str($d_check['type'], $d_check['key_'], $d_check['ports']);
+					NAME_DELIMITER.discovery_check2str($d_check['type'], $d_check['key_'], $d_check['ports'],
+						$d_check['allow_redirect']
+					);
 
 				$values = [
 					$options['dstfld1'] => $d_check[$options['srcfld1']]
@@ -447,7 +488,6 @@ switch ($data['popup_type']) {
 	case 'items':
 	case 'template_items':
 	case 'item_prototypes':
-
 		if ($options['srcfld2'] !== '' && $options['dstfld2'] !== '') {
 			// TODO: this condition must be removed after all item and item_prototype fields changing to multiselect
 			foreach ($data['table_records'] as &$item) {
@@ -495,8 +535,16 @@ switch ($data['popup_type']) {
 
 				$table->addRow([
 					$data['multiselect'] ? new CCheckBox('item['.$checkbox_key.']', $item['itemid']) : null,
-					$description,
-					(new CDiv($item['key_']))->addClass(ZBX_STYLE_WORDWRAP),
+					(new CDiv($description))
+						->addClass(ZBX_STYLE_WORDBREAK)
+						->addClass(ZBX_STYLE_LINE_CLAMP)
+						->addStyle('--line-clamp: 4;')
+						->setTitle($item['name']),
+					(new CDiv($item['key_']))
+						->addClass(ZBX_STYLE_WORDWRAP)
+						->addClass(ZBX_STYLE_LINE_CLAMP)
+						->addStyle('--line-clamp: 4;')
+						->setTitle($item['key_']),
 					item_type2str($item['type']),
 					itemValueTypeString($item['value_type']),
 					($data['popup_type'] === 'items')
@@ -529,16 +577,25 @@ switch ($data['popup_type']) {
 					$data['multiselect']
 						? new CCheckBox('item['.$item['itemid'].']', $item_pattern)
 						: null,
-					(new CLink($item['name']))
-						->setAttribute('data-reference', $options['reference'])
-						->setAttribute('data-pattern', $item_pattern)
-						->setAttribute('data-parentid', $options['parentid'])
-						->onClick('
-							addValue(this.dataset.reference, this.dataset.pattern, this.dataset.parentid ?? null);
-							popup_generic.closePopup(event);
-						')
-						->addClass(ZBX_STYLE_WORDBREAK),
-					(new CDiv($item['key_']))->addClass(ZBX_STYLE_WORDWRAP),
+					(new CDiv(
+						(new CLink($item['name']))
+							->setAttribute('data-reference', $options['reference'])
+							->setAttribute('data-pattern', $item_pattern)
+							->setAttribute('data-parentid', $options['parentid'])
+							->onClick('
+								addValue(this.dataset.reference, this.dataset.pattern, this.dataset.parentid ?? null);
+								popup_generic.closePopup(event);
+							')
+					))
+						->addClass(ZBX_STYLE_WORDBREAK)
+						->addClass(ZBX_STYLE_LINE_CLAMP)
+						->addStyle('--line-clamp: 4;')
+						->setTitle($item['name']),
+					(new CDiv($item['key_']))
+						->addClass(ZBX_STYLE_WORDWRAP)
+						->addClass(ZBX_STYLE_LINE_CLAMP)
+						->addStyle('--line-clamp: 4;')
+						->setTitle($item['key_']),
 					item_type2str($item['type']),
 					itemValueTypeString($item['value_type']),
 					($data['popup_type'] === 'items')
@@ -856,6 +913,7 @@ $types = [
 	'item_prototypes',
 	'media_types',
 	'proxies',
+	'proxy_groups',
 	'roles',
 	'templates',
 	'users',
@@ -879,7 +937,7 @@ $output['script_inline'] = $this->readJsFile('popup.generic.js.php').
 
 if ($form) {
 	// Enable form submitting on Enter.
-	$form->addItem((new CSubmitButton(null))->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
+	$form->addItem((new CSubmitButton())->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
 
 	$form->addItem($table);
 
