@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -38,7 +33,7 @@ $operations_popup_form = (new CForm())
 	->addVar('action', 'popup.lldoperation');
 
 // Enable form submitting on Enter.
-$operations_popup_form->addItem((new CSubmitButton(null))->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
+$operations_popup_form->addItem((new CSubmitButton())->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
 
 $operations_popup_form_list = (new CFormList())
 	->addRow(
@@ -172,13 +167,13 @@ $operations_popup_form_list
 	)
 	->addRow(
 		(new CVisibilityBox('visible[ophistory]', 'ophistory-field', _('Original')))
-			->setLabel(_('History storage period'))
+			->setLabel(_('History'))
 			->setChecked(array_key_exists('ophistory', $options))
 			->setReadonly($options['templated']),
 		(new CDiv([
 			(new CRadioButtonList('ophistory[history_mode]', (int) $field_values['ophistory']['history_mode']))
-				->addValue(_('Do not keep history'), ITEM_STORAGE_OFF)
-				->addValue(_('Storage period'), ITEM_STORAGE_CUSTOM)
+				->addValue(_('Do not store'), ITEM_STORAGE_OFF)
+				->addValue(_('Store up to'), ITEM_STORAGE_CUSTOM)
 				->setModern(true)
 				->setReadonly($options['templated']),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -193,13 +188,13 @@ $operations_popup_form_list
 	)
 	->addRow(
 		(new CVisibilityBox('visible[optrends]', 'optrends-field', _('Original')))
-			->setLabel(_('Trend storage period'))
+			->setLabel(_('Trends'))
 			->setChecked(array_key_exists('optrends', $options))
 			->setReadonly($options['templated']),
 		(new CDiv([
 			(new CRadioButtonList('optrends[trends_mode]', (int) $field_values['optrends']['trends_mode']))
-				->addValue(_('Do not keep trends'), ITEM_STORAGE_OFF)
-				->addValue(_('Storage period'), ITEM_STORAGE_CUSTOM)
+				->addValue(_('Do not store'), ITEM_STORAGE_OFF)
+				->addValue(_('Store up to'), ITEM_STORAGE_CUSTOM)
 				->setModern(true)
 				->setReadonly($options['templated']),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
