@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -31,88 +26,160 @@ abstract class CHostBase extends CApiService {
 	protected $tableName = 'hosts';
 	protected $tableAlias = 'h';
 
-	/**
-	 * Check for valid templates.
-	 *
-	 * @param array      $hosts
-	 * @param array|null $db_hosts
-	 *
-	 * @throws APIException
-	 */
-	protected function checkTemplates(array $hosts, array $db_hosts = null): void {
+	protected function checkTemplates(array &$hosts, ?array &$db_hosts = null, ?string $path = null,
+			?array $template_indexes = null, ?string $path_clear = null, ?array $template_clear_indexes = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
-		$edit_templates = [];
+		$ins_template_indexes = [];
+		$clear_template_indexes = [];
 
-		foreach ($hosts as $i1 => $host) {
+		foreach ($hosts as $i1 => &$host) {
 			if (array_key_exists('templates', $host) && array_key_exists('templates_clear', $host)) {
-				$path_clear = '/'.($i1 + 1).'/templates_clear';
-				$path = '/'.($i1 + 1).'/templates';
-
 				foreach ($host['templates_clear'] as $i2_clear => $template_clear) {
 					foreach ($host['templates'] as $i2 => $template) {
-						if (bccomp($template['templateid'], $template_clear['templateid']) == 0) {
-							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
-								$path_clear.'/'.($i2_clear + 1).'/templateid',
-								_s('cannot be specified the value of parameter "%1$s"',
-									$path.'/'.($i2 + 1).'/templateid'
-								)
-							));
+						if (bccomp($template['templateid'], $template_clear['templateid']) != 0) {
+							continue;
 						}
+
+						if ($path === null) {
+							$path_clear = '/'.($i1 + 1).'/templates_clear/'.($i2_clear + 1);
+							$path = '/'.($i1 + 1).'/templates/'.($i2 + 1);
+						}
+						else {
+							$path_clear .= '/'.($template_clear_indexes[$template['templateid']] + 1);
+							$path .= '/'.($template_indexes[$template['templateid']] + 1);
+						}
+
+						$error = _s('cannot be specified the value of parameter "%1$s"', $path.'/templateid');
+
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Invalid parameter "%1$s": %2$s.', $path_clear.'/templateid', $error)
+						);
 					}
 				}
 			}
 
 			if (array_key_exists('templates', $host)) {
-				$templates = array_column($host['templates'], null, 'templateid');
+				$db_templates = $db_hosts !== null
+					? array_column($db_hosts[$host[$id_field_name]]['templates'], null, 'templateid')
+					: [];
 
-				if ($db_hosts === null) {
-					$edit_templates += $templates;
+				foreach ($host['templates'] as $i2 => $template) {
+					if (array_key_exists($template['templateid'], $db_templates)) {
+						if ($db_templates[$template['templateid']]['link_type'] != TEMPLATE_LINK_MANUAL) {
+							unset($host['templates'][$i2]);
+
+							$db_hosttemplateid = $db_templates[$template['templateid']]['hosttemplateid'];
+							unset($db_hosts[$host[$id_field_name]]['templates'][$db_hosttemplateid]);
+						}
+
+						unset($db_templates[$template['templateid']]);
+					}
+					else {
+						$ins_template_indexes[$template['templateid']][$i1] = $i2;
+					}
 				}
-				else {
-					$db_templates = array_column($db_hosts[$host[$id_field_name]]['templates'], null, 'templateid');
 
-					$ins_templates = array_diff_key($templates, $db_templates);
-					$del_templates = array_diff_key($db_templates, $templates);
+				if ($db_templates) {
+					$templateids_clear = array_key_exists('templates_clear', $host)
+						? array_column($host['templates_clear'], 'templateid')
+						: [];
 
-					$edit_templates += $ins_templates + $del_templates;
+					foreach ($db_templates as $db_template) {
+						if ($db_template['link_type'] != TEMPLATE_LINK_MANUAL
+								&& !in_array($db_template['templateid'], $templateids_clear)) {
+							unset($db_hosts[$host[$id_field_name]]['templates'][$db_template['hosttemplateid']]);
+						}
+					}
 				}
 			}
 
 			if (array_key_exists('templates_clear', $host)) {
-				$edit_templates += array_column($host['templates_clear'], null, 'templateid');
+				$db_templates = array_column($db_hosts[$host[$id_field_name]]['templates'], null, 'templateid');
+
+				foreach ($host['templates_clear'] as $i2 => $template) {
+					if (array_key_exists($template['templateid'], $db_templates)) {
+						if ($db_templates[$template['templateid']]['link_type'] != TEMPLATE_LINK_MANUAL) {
+							unset($host['templates_clear'][$i2]);
+
+							$db_hosttemplateid = $db_templates[$template['templateid']]['hosttemplateid'];
+							unset($db_hosts[$host[$id_field_name]]['templates'][$db_hosttemplateid]);
+						}
+
+						unset($db_templates[$template['templateid']]);
+					}
+					else {
+						$clear_template_indexes[$template['templateid']][$i1] = $i2;
+					}
+				}
 			}
 		}
+		unset($host);
 
-		if (!$edit_templates) {
-			return;
-		}
+		if ($ins_template_indexes) {
+			$db_templates = API::Template()->get([
+				'output' => [],
+				'templateids' => array_keys($ins_template_indexes),
+				'preservekeys' => true
+			]);
 
-		$count = API::Template()->get([
-			'countOutput' => true,
-			'templateids' => array_keys($edit_templates)
-		]);
+			foreach ($ins_template_indexes as $templateid => $indexes) {
+				if (!array_key_exists($templateid, $db_templates)) {
+					if ($path === null) {
+						$i1 = key($indexes);
+						$i2 = $indexes[$i1];
 
-		if ($count != count($edit_templates)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
+						$path = '/'.($i1 + 1).'/templates/'.($i2 + 1);
+					}
+					else {
+						$i = $template_indexes[$templateid];
 
-		foreach ($hosts as $i1 => $host) {
-			if (!array_key_exists('templates_clear', $host)) {
-				continue;
-			}
+						$path .= '/'.($i + 1);
+					}
 
-			$db_templates = array_column($db_hosts[$host[$id_field_name]]['templates'], null, 'templateid');
-			$path = '/'.($i1 + 1).'/templates_clear';
-
-			foreach ($host['templates_clear'] as $i2 => $template) {
-				if (!array_key_exists($template['templateid'], $db_templates)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
-						$path.'/'.($i2 + 1).'/templateid', _('cannot be unlinked')
+					self::exception(ZBX_API_ERROR_PERMISSIONS, _s('Invalid parameter "%1$s": %2$s.', $path,
+						_('object does not exist, or you have no permissions to it')
 					));
 				}
 			}
 		}
+
+		if ($clear_template_indexes) {
+			$db_templates = API::Template()->get([
+				'output' => [],
+				'templateids' => array_keys($clear_template_indexes),
+				'preservekeys' => true
+			]);
+
+			foreach ($clear_template_indexes as $templateid => $indexes) {
+				if (!array_key_exists($templateid, $db_templates)) {
+					if ($path_clear === null) {
+						$i1 = key($indexes);
+						$i2 = $indexes[$i1];
+
+						$path_clear = '/'.($i1 + 1).'/templates_clear/'.($i2 + 1);
+					}
+					else {
+						$i = $template_clear_indexes[$template['templateid']];
+
+						$path_clear .= '/'.($i + 1);
+					}
+
+					self::exception(ZBX_API_ERROR_PERMISSIONS, _s('Invalid parameter "%1$s": %2$s.', $path_clear,
+						_('object does not exist, or you have no permissions to it')
+					));
+				}
+				else {
+					foreach ($indexes as $i1 => $i2) {
+						unset($hosts[$i1]['templates_clear'][$i2]);
+
+						if (!$hosts[$i1]['templates_clear']) {
+							unset($hosts[$i1]['templates_clear']);
+						}
+					}
+				}
+			}
+		}
 	}
 
 	/**
@@ -121,7 +188,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array      $hosts
 	 * @param array|null $db_hosts
 	 */
-	protected function checkTemplatesLinks(array $hosts, array $db_hosts = null): void {
+	protected function checkTemplatesLinks(array $hosts, ?array $db_hosts = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$ins_templates = [];
@@ -625,7 +692,7 @@ abstract class CHostBase extends CApiService {
 	 * @return array|null
 	 */
 	private static function getScopedInsTemplates(array $ins_templates, ?array $scope,
-			array &$db_templates = null): array {
+			?array &$db_templates = null): array {
 		$scoped_ins_templates = [];
 		$db_templates = [];
 
@@ -1032,7 +1099,7 @@ abstract class CHostBase extends CApiService {
 	 * @return array
 	 */
 	private static function getTemplateOrTargetRelatedIds(array $links, array $sourceids,
-			string $ignore_relatedid = null): array {
+			?string $ignore_relatedid = null): array {
 		$processed_sourceids = $sourceids;
 		$relatedids = [];
 
@@ -1279,7 +1346,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array|null $db_hosts
 	 * @param array|null $upd_hostids
 	 */
-	protected function updateTemplates(array &$hosts, array &$db_hosts = null, array &$upd_hostids = null): void {
+	protected function updateTemplates(array &$hosts, ?array &$db_hosts = null, ?array &$upd_hostids = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$ins_hosts_templates = [];
@@ -1342,7 +1409,7 @@ abstract class CHostBase extends CApiService {
 					$upd_hostids[$i] = $host[$id_field_name];
 				}
 				else {
-					unset($host['templates'], $host['templates_clear'], $db_hosts[$host[$id_field_name]]['templates']);
+					unset($host['templates'], $db_hosts[$host[$id_field_name]]['templates']);
 				}
 			}
 		}
@@ -1376,7 +1443,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array|null $db_hosts
 	 * @param array|null $upd_hostids
 	 */
-	protected function updateTags(array &$hosts, array &$db_hosts = null, array &$upd_hostids = null): void {
+	protected function updateTags(array &$hosts, ?array &$db_hosts = null, ?array &$upd_hostids = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$ins_tags = [];
@@ -1457,7 +1524,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array|null $db_hosts
 	 * @param array|null $upd_hostids
 	 */
-	protected function updateMacros(array &$hosts, array &$db_hosts = null, array &$upd_hostids = null): void {
+	protected function updateMacros(array &$hosts, ?array &$db_hosts = null, ?array &$upd_hostids = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$ins_hostmacros = [];
@@ -1536,399 +1603,6 @@ abstract class CHostBase extends CApiService {
 		unset($host);
 	}
 
-	/**
-	 * Links the templates to the given hosts.
-	 *
-	 * @param array $templateIds
-	 * @param array $targetIds		an array of host IDs to link the templates to
-	 *
-	 * @return array 	an array of added hosts_templates rows, with 'hostid' and 'templateid' set for each row
-	 */
-	protected function link(array $templateIds, array $targetIds) {
-		if (empty($templateIds)) {
-			return;
-		}
-
-		// check if someone passed duplicate templates in the same query
-		$templateIdDuplicates = zbx_arrayFindDuplicates($templateIds);
-		if ($templateIdDuplicates) {
-			$duplicatesFound = [];
-			foreach ($templateIdDuplicates as $value => $count) {
-				$duplicatesFound[] = _s('template ID "%1$s" is passed %2$s times', $value, $count);
-			}
-			self::exception(
-				ZBX_API_ERROR_PARAMETERS,
-				_s('Cannot pass duplicate template IDs for the linkage: %1$s.', implode(', ', $duplicatesFound))
-			);
-		}
-
-		$count = API::Template()->get([
-			'countOutput' => true,
-			'templateids' => $templateIds
-		]);
-
-		if ($count != count($templateIds)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
-		// get DB templates which exists in all targets
-		$res = DBselect('SELECT * FROM hosts_templates WHERE '.dbConditionInt('hostid', $targetIds));
-		$mas = [];
-		while ($row = DBfetch($res)) {
-			if (!isset($mas[$row['templateid']])) {
-				$mas[$row['templateid']] = [];
-			}
-			$mas[$row['templateid']][$row['hostid']] = 1;
-		}
-		$commonDBTemplateIds = [];
-		foreach ($mas as $templateId => $targetList) {
-			if (count($targetList) == count($targetIds)) {
-				$commonDBTemplateIds[] = $templateId;
-			}
-		}
-
-		// check if there are any template with triggers which depends on triggers in templates which will be not linked
-		$commonTemplateIds = array_unique(array_merge($commonDBTemplateIds, $templateIds));
-		foreach ($templateIds as $templateid) {
-			$triggerids = [];
-			$dbTriggers = get_triggers_by_hostid($templateid);
-			while ($trigger = DBfetch($dbTriggers)) {
-				$triggerids[$trigger['triggerid']] = $trigger['triggerid'];
-			}
-
-			$sql = 'SELECT DISTINCT h.host'.
-				' FROM trigger_depends td,functions f,items i,hosts h'.
-				' WHERE ('.
-				dbConditionInt('td.triggerid_down', $triggerids).
-				' AND f.triggerid=td.triggerid_up'.
-				' )'.
-				' AND i.itemid=f.itemid'.
-				' AND h.hostid=i.hostid'.
-				' AND '.dbConditionInt('h.hostid', $commonTemplateIds, true).
-				' AND h.status='.HOST_STATUS_TEMPLATE;
-			if ($dbDepHost = DBfetch(DBselect($sql))) {
-				$tmpTpls = API::Template()->get([
-					'output'=> ['host'],
-					'templateids' => $templateid
-				]);
-
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Trigger in template "%1$s" has dependency with trigger in template "%2$s".', $tmpTpls[0]['host'], $dbDepHost['host']));
-			}
-		}
-
-		$res = DBselect(
-			'SELECT ht.hostid,ht.templateid'.
-				' FROM hosts_templates ht'.
-				' WHERE '.dbConditionInt('ht.hostid', $targetIds).
-				' AND '.dbConditionInt('ht.templateid', $templateIds)
-		);
-		$linked = [];
-		while ($row = DBfetch($res)) {
-			$linked[$row['templateid']][$row['hostid']] = true;
-		}
-
-		// add template linkages, if problems rollback later
-		$hostsLinkageInserts = [];
-
-		foreach ($templateIds as $templateid) {
-			$linked_targets = array_key_exists($templateid, $linked) ? $linked[$templateid] : [];
-
-			foreach ($targetIds as $targetid) {
-				if (array_key_exists($targetid, $linked_targets)) {
-					continue;
-				}
-
-				$hostsLinkageInserts[] = ['hostid' => $targetid, 'templateid' => $templateid];
-			}
-		}
-
-		if ($hostsLinkageInserts) {
-			self::checkCircularLinkage($hostsLinkageInserts);
-			self::checkDoubleLinkage($hostsLinkageInserts);
-
-			$hosttemplateids = DB::insertBatch('hosts_templates', $hostsLinkageInserts);
-
-			foreach ($hostsLinkageInserts as &$host_linkage) {
-				$host_linkage['hosttemplateid'] = array_shift($hosttemplateids);
-			}
-			unset($host_linkage);
-		}
-
-		// check if all trigger templates are linked to host.
-		// we try to find template that is not linked to hosts ($targetids)
-		// and exists trigger which reference that template and template from ($templateids)
-		$sql = 'SELECT DISTINCT h.host'.
-			' FROM functions f,items i,triggers t,hosts h'.
-			' WHERE f.itemid=i.itemid'.
-			' AND f.triggerid=t.triggerid'.
-			' AND i.hostid=h.hostid'.
-			' AND h.status='.HOST_STATUS_TEMPLATE.
-			' AND NOT EXISTS (SELECT 1 FROM hosts_templates ht WHERE ht.templateid=i.hostid AND '.dbConditionInt('ht.hostid', $targetIds).')'.
-			' AND EXISTS (SELECT 1 FROM functions ff,items ii WHERE ff.itemid=ii.itemid AND ff.triggerid=t.triggerid AND '.dbConditionInt('ii.hostid', $templateIds). ')';
-		if ($dbNotLinkedTpl = DBfetch(DBSelect($sql, 1))) {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Trigger has items from template "%1$s" that is not linked to host.', $dbNotLinkedTpl['host'])
-			);
-		}
-
-		return $hostsLinkageInserts;
-	}
-
-	protected function unlink($templateids, $targetids = null) {
-		$cond = ['templateid' => $templateids];
-		if (!is_null($targetids)) {
-			$cond['hostid'] = $targetids;
-		}
-		DB::delete('hosts_templates', $cond);
-
-		if (!is_null($targetids)) {
-			$hosts = API::Host()->get([
-				'hostids' => $targetids,
-				'output' => ['hostid', 'host'],
-				'nopermissions' => true
-			]);
-		}
-		else {
-			$hosts = API::Host()->get([
-				'templateids' => $templateids,
-				'output' => ['hostid', 'host'],
-				'nopermissions' => true
-			]);
-		}
-
-		if (!empty($hosts)) {
-			$templates = API::Template()->get([
-				'templateids' => $templateids,
-				'output' => ['hostid', 'host'],
-				'nopermissions' => true
-			]);
-
-			$hosts = implode(', ', zbx_objectValues($hosts, 'host'));
-			$templates = implode(', ', zbx_objectValues($templates, 'host'));
-
-			info(_s('Templates "%1$s" unlinked from hosts "%2$s".', $templates, $hosts));
-		}
-	}
-
-	/**
-	 * Searches for circular linkages.
-	 *
-	 * @param array  $host_templates
-	 * @param string $host_templates[]['templateid']
-	 * @param string $host_templates[]['hostid']
-	 */
-	private static function checkCircularLinkage(array $host_templates) {
-		$links = [];
-
-		foreach ($host_templates as $host_template) {
-			$links[$host_template['templateid']][$host_template['hostid']] = true;
-		}
-
-		$templateids = array_keys($links);
-		$_templateids = $templateids;
-
-		do {
-			$result = DBselect(
-				'SELECT ht.templateid,ht.hostid'.
-				' FROM hosts_templates ht'.
-				' WHERE '.dbConditionId('ht.hostid', $_templateids)
-			);
-
-			$_templateids = [];
-
-			while ($row = DBfetch($result)) {
-				if (!array_key_exists($row['templateid'], $links)) {
-					$_templateids[$row['templateid']] = true;
-				}
-
-				$links[$row['templateid']][$row['hostid']] = true;
-			}
-
-			$_templateids = array_keys($_templateids);
-		}
-		while ($_templateids);
-
-		foreach ($templateids as $templateid) {
-			self::checkTemplateCircularLinkage($links, $templateid, $links[$templateid]);
-		}
-	}
-
-	/**
-	 * Searches for circular linkages for specific template.
-	 *
-	 * @param array  $links[<templateid>][<hostid>]  The list of linkages.
-	 * @param string $templateid                     ID of the template to check circular linkages.
-	 * @param array  $hostids[<hostid>]
-	 *
-	 * @throws APIException if circular linkage is found.
-	 */
-	private static function checkTemplateCircularLinkage(array $links, $templateid, array $hostids): void {
-		if (array_key_exists($templateid, $hostids)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Circular template linkage is not allowed.'));
-		}
-
-		foreach ($hostids as $hostid => $foo) {
-			if (array_key_exists($hostid, $links)) {
-				self::checkTemplateCircularLinkage($links, $templateid, $links[$hostid]);
-			}
-		}
-	}
-
-	/**
-	 * Searches for double linkages.
-	 *
-	 * @param array  $host_templates
-	 * @param string $host_templates[]['templateid']
-	 * @param string $host_templates[]['hostid']
-	 */
-	private static function checkDoubleLinkage(array $host_templates) {
-		$links = [];
-		$templateids = [];
-		$hostids = [];
-
-		foreach ($host_templates as $host_template) {
-			$links[$host_template['hostid']][$host_template['templateid']] = true;
-			$templateids[$host_template['templateid']] = true;
-			$hostids[$host_template['hostid']] = true;
-		}
-
-		$_hostids = array_keys($hostids);
-
-		do {
-			$result = DBselect(
-				'SELECT ht.hostid'.
-				' FROM hosts_templates ht'.
-				' WHERE '.dbConditionId('ht.templateid', $_hostids)
-			);
-
-			$_hostids = [];
-
-			while ($row = DBfetch($result)) {
-				if (!array_key_exists($row['hostid'], $hostids)) {
-					$_hostids[$row['hostid']] = true;
-				}
-
-				$hostids[$row['hostid']] = true;
-			}
-
-			$_hostids = array_keys($_hostids);
-		}
-		while ($_hostids);
-
-		$_templateids = array_keys($templateids + $hostids);
-		$templateids = [];
-
-		do {
-			$result = DBselect(
-				'SELECT ht.templateid,ht.hostid'.
-				' FROM hosts_templates ht'.
-				' WHERE '.dbConditionId('hostid', $_templateids)
-			);
-
-			$_templateids = [];
-
-			while ($row = DBfetch($result)) {
-				if (!array_key_exists($row['templateid'], $templateids)) {
-					$_templateids[$row['templateid']] = true;
-				}
-
-				$templateids[$row['templateid']] = true;
-				$links[$row['hostid']][$row['templateid']] = true;
-			}
-
-			$_templateids = array_keys($_templateids);
-		}
-		while ($_templateids);
-
-		foreach ($hostids as $hostid => $foo) {
-			self::checkTemplateDoubleLinkage($links, $hostid);
-		}
-	}
-
-	/**
-	 * Searches for double linkages.
-	 *
-	 * @param array  $links[<hostid>][<templateid>]  The list of linked template IDs by host ID.
-	 * @param string $hostid
-	 *
-	 * @throws APIException if double linkage is found.
-	 *
-	 * @return array  An array of the linked templates for the selected host.
-	 */
-	private static function checkTemplateDoubleLinkage(array $links, $hostid): array {
-		$templateids = $links[$hostid];
-
-		foreach ($links[$hostid] as $templateid => $foo) {
-			if (array_key_exists($templateid, $links)) {
-				$_templateids = self::checkTemplateDoubleLinkage($links, $templateid);
-
-				if (array_intersect_key($templateids, $_templateids)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_('Template cannot be linked to another template more than once even through other templates.')
-					);
-				}
-
-				$templateids += $_templateids;
-			}
-		}
-
-		return $templateids;
-	}
-
-	/**
-	 * Creates user macros for hosts, templates and host prototypes.
-	 *
-	 * @param array  $hosts
-	 * @param array  $hosts[]['templateid|hostid']
-	 * @param array  $hosts[]['macros']             (optional)
-	 */
-	protected function createHostMacros(array $hosts): void {
-		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
-
-		$ins_hostmacros = [];
-
-		foreach ($hosts as $host) {
-			if (array_key_exists('macros', $host)) {
-				foreach ($host['macros'] as $macro) {
-					$ins_hostmacros[] = ['hostid' => $host[$id_field_name]] + $macro;
-				}
-			}
-		}
-
-		if ($ins_hostmacros) {
-			DB::insert('hostmacro', $ins_hostmacros);
-		}
-	}
-
-	/**
-	 * Adding "macros" to the each host object.
-	 *
-	 * @param array  $db_hosts
-	 *
-	 * @return array
-	 */
-	protected function getHostMacros(array $db_hosts): array {
-		foreach ($db_hosts as &$db_host) {
-			$db_host['macros'] = [];
-		}
-		unset($db_host);
-
-		$options = [
-			'output' => ['hostmacroid', 'hostid', 'macro', 'type', 'value', 'description', 'automatic'],
-			'filter' => ['hostid' => array_keys($db_hosts)]
-		];
-		$db_macros = DBselect(DB::makeSql('hostmacro', $options));
-
-		while ($db_macro = DBfetch($db_macros)) {
-			$hostid = $db_macro['hostid'];
-			unset($db_macro['hostid']);
-
-			$db_hosts[$hostid]['macros'][$db_macro['hostmacroid']] = $db_macro;
-		}
-
-		return $db_hosts;
-	}
-
 	/**
 	 * Checks user macros for host.update, template.update and hostprototype.update methods.
 	 *
@@ -2048,63 +1722,6 @@ abstract class CHostBase extends CApiService {
 		return $hosts;
 	}
 
-	/**
-	 * Updates user macros for hosts, templates and host prototypes.
-	 *
-	 * @param array  $hosts
-	 * @param array  $hosts[]['templateid|hostid']
-	 * @param array  $hosts[]['macros']             (optional)
-	 * @param array  $db_hosts
-	 * @param array  $db_hosts[<hostid>]['macros']  An array of host macros indexed by hostmacroid.
-	 */
-	protected function updateHostMacros(array $hosts, array $db_hosts): void {
-		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
-
-		$ins_hostmacros = [];
-		$upd_hostmacros = [];
-		$del_hostmacroids = [];
-
-		foreach ($hosts as $host) {
-			if (!array_key_exists('macros', $host)) {
-				continue;
-			}
-
-			$db_host = $db_hosts[$host[$id_field_name]];
-
-			foreach ($host['macros'] as $hostmacro) {
-				if (array_key_exists('hostmacroid', $hostmacro)) {
-					$db_hostmacro = $db_host['macros'][$hostmacro['hostmacroid']];
-					unset($db_host['macros'][$hostmacro['hostmacroid']]);
-
-					$upd_hostmacro = DB::getUpdatedValues('hostmacro', $hostmacro, $db_hostmacro);
-
-					if ($upd_hostmacro) {
-						$upd_hostmacros[] = [
-							'values' => $upd_hostmacro,
-							'where' => ['hostmacroid' => $hostmacro['hostmacroid']]
-						];
-					}
-				}
-				else {
-					$ins_hostmacros[] = $hostmacro + ['hostid' => $host[$id_field_name]];
-				}
-			}
-
-			$del_hostmacroids = array_merge($del_hostmacroids, array_keys($db_host['macros']));
-		}
-
-		if ($del_hostmacroids) {
-			DB::delete('hostmacro', ['hostmacroid' => $del_hostmacroids]);
-		}
-
-		if ($upd_hostmacros) {
-			DB::update('hostmacro', $upd_hostmacros);
-		}
-
-		if ($ins_hostmacros) {
-			DB::insert('hostmacro', $ins_hostmacros);
-		}
-	}
 
 	/**
 	 * @param array $hosts
@@ -2120,7 +1737,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array $hosts
 	 * @param array $db_hosts
 	 */
-	private function addAffectedTemplates(array $hosts, array &$db_hosts): void {
+	protected function addAffectedTemplates(array $hosts, array &$db_hosts): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$hostids = [];
@@ -2138,7 +1755,7 @@ abstract class CHostBase extends CApiService {
 
 		$permitted_templates = [];
 
-		if (self::$userData['type'] == USER_TYPE_ZABBIX_ADMIN) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
 			$permitted_templates = API::Template()->get([
 				'output' => [],
 				'hostids' => $hostids,
@@ -2147,7 +1764,7 @@ abstract class CHostBase extends CApiService {
 		}
 
 		$options = [
-			'output' => ['hosttemplateid', 'hostid', 'templateid'],
+			'output' => ['hosttemplateid', 'hostid', 'templateid', 'link_type'],
 			'filter' => ['hostid' => $hostids]
 		];
 		$db_templates = DBselect(DB::makeSql('hosts_templates', $options));
@@ -2201,7 +1818,7 @@ abstract class CHostBase extends CApiService {
 	 * @param array $hosts
 	 * @param array $db_hosts
 	 */
-	private function addAffectedMacros(array $hosts, array &$db_hosts): void {
+	protected function addAffectedMacros(array $hosts, array &$db_hosts): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		$hostids = [];
@@ -2218,7 +1835,7 @@ abstract class CHostBase extends CApiService {
 		}
 
 		$options = [
-			'output' => ['hostmacroid', 'hostid', 'macro', 'value', 'description', 'type'],
+			'output' => ['hostmacroid', 'hostid', 'macro', 'value', 'description', 'type', 'automatic'],
 			'filter' => ['hostid' => $hostids]
 		];
 		$db_macros = DBselect(DB::makeSql('hostmacro', $options));
