@@ -588,6 +588,8 @@ class CUserDirectory extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
+		self::addBindPassword($db_userdirectories);
+
 		foreach ($userdirectories as $i => &$userdirectory) {
 			$db_userdirectory = $db_userdirectories[$userdirectory['userdirectoryid']];
 			$userdirectory += [
@@ -618,6 +620,19 @@ class CUserDirectory extends CApiService {
 		self::checkDuplicates($userdirectories, $db_userdirectories);
 		self::checkProvisionGroups($userdirectories, $db_userdirectories);
 		self::checkMediaTypes($userdirectories, $db_userdirectories);
+		self::checkLdapBindPassword($userdirectories, $db_userdirectories);
+	}
+
+	private static function addBindPassword(array &$db_userdirectories): void {
+		$db_bind_passwords = DB::select('userdirectory_ldap', [
+			'output' => ['bind_password'],
+			'filter' => ['userdirectoryid' => array_keys($db_userdirectories)],
+			'preservekeys' => true
+		]);
+
+		foreach ($db_bind_passwords as $db_userdirectoryid => $db_bind_password) {
+			$db_userdirectories[$db_userdirectoryid] += $db_bind_password;
+		}
 	}
 
 	private static function addRequiredFieldsByType(array &$userdirectories, array $db_userdirectories): void {
@@ -944,6 +959,24 @@ class CUserDirectory extends CApiService {
 		}
 	}
 
+	private static function checkLdapBindPassword(array $userdirectories, array $db_userdirectories): void {
+		foreach ($userdirectories as $i => $userdirectory) {
+			if ($userdirectory['idp_type'] != IDP_TYPE_LDAP) {
+				continue;
+			}
+
+			$db_userdirectory = $db_userdirectories[$userdirectory['userdirectoryid']];
+
+			if (array_key_exists('host', $userdirectory) && $userdirectory['host'] !== $db_userdirectory['host']
+					&& !array_key_exists('bind_password', $userdirectory)
+					&& $db_userdirectory['bind_password'] !== '') {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1),
+					_s('the parameter "%1$s" is missing', 'bind_password')
+				));
+			}
+		}
+	}
+
 	private static function addFieldDefaultsByType(array &$userdirectories, array $db_userdirectories): void {
 		foreach ($userdirectories as &$userdirectory) {
 			$db_userdirectory = $db_userdirectories[$userdirectory['userdirectoryid']];
