@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -35,6 +30,12 @@ class CUserGroup extends CApiService {
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['usrgrpid', 'name'];
 
+	public const OUTPUT_FIELDS = ['usrgrpid', 'name', 'gui_access', 'users_status', 'debug_mode', 'userdirectoryid',
+		'mfa_status', 'mfaid'
+	];
+
+	public const LIMITED_OUTPUT_FIELDS = ['usrgrpid', 'name', 'gui_access', 'users_status', 'debug_mode', 'mfa_status'];
+
 	/**
 	 * Get user groups.
 	 *
@@ -64,7 +65,9 @@ class CUserGroup extends CApiService {
 		$defOptions = [
 			'usrgrpids'					=> null,
 			'userids'					=> null,
+			'mfaids'					=> null,
 			'status'					=> null,
+			'mfa_status'				=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> null,
@@ -105,6 +108,20 @@ class CUserGroup extends CApiService {
 			}
 		}
 
+		// output
+		if (!$options['countOutput']) {
+			$usrgrps_output_fields = self::$userData['type'] == USER_TYPE_SUPER_ADMIN
+				? self::OUTPUT_FIELDS
+				: self::LIMITED_OUTPUT_FIELDS;
+
+			if (is_array($options['output'])) {
+				$options['output'] = array_intersect($options['output'], $usrgrps_output_fields);
+			}
+			elseif ($options['output'] === API_OUTPUT_EXTEND) {
+				$options['output'] = $usrgrps_output_fields;
+			}
+		}
+
 		// usrgrpids
 		if (!is_null($options['usrgrpids'])) {
 			zbx_value2array($options['usrgrpids']);
@@ -121,13 +138,47 @@ class CUserGroup extends CApiService {
 			$sqlParts['where']['gug'] = 'g.usrgrpid=ug.usrgrpid';
 		}
 
+		if (!is_null($options['mfaids'])) {
+			if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+				return [];
+			}
+
+			zbx_value2array($options['mfaids']);
+
+			$sqlParts['where'][] = dbConditionId('g.mfaid', $options['mfaids']);
+		}
+
 		// status
 		if (!is_null($options['status'])) {
 			$sqlParts['where'][] = 'g.users_status='.zbx_dbstr($options['status']);
 		}
 
+		if (!is_null($options['mfa_status'])) {
+			zbx_value2array($options['mfa_status']);
+
+			$sqlParts['where'][] = dbConditionInt('g.mfa_status', $options['mfa_status']);
+		}
+
+		$limited_output_fields = array_flip(self::LIMITED_OUTPUT_FIELDS);
+
 		// filter
 		if (is_array($options['filter'])) {
+			if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+				if ($options['searchByAny'] !== null && $options['searchByAny'] !== false) {
+					$options['filter'] = array_diff_key($options['filter'], array_flip(['userdirectoryid', 'mfaid']));
+				}
+				else {
+					$has_private_fields = array_intersect_key(
+						array_diff_key($options['filter'], $limited_output_fields),
+						array_flip(['userdirectoryid', 'mfaid'])
+					);
+
+					if($has_private_fields) {
+						return [];
+					}
+				}
+			}
+
 			$this->dbFilter('usrgrp g', $options, $sqlParts);
 		}
 
@@ -222,6 +273,11 @@ class CUserGroup extends CApiService {
 											['if' => ['field' => 'gui_access', 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_LDAP])], 'type' => API_ID],
 											['else' => true, 'type' => API_UNEXPECTED]
 			]],
+			'mfa_status' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_MFA_DISABLED, GROUP_MFA_ENABLED]), 'default' => DB::getDefault('usrgrp', 'mfa_status')],
+			'mfaid' =>					['type' => API_MULTIPLE, 'rules' => [
+											['if' => ['field' => 'mfa_status', 'in' => implode(',', [GROUP_MFA_ENABLED])], 'type' => API_ID],
+											['else' => true, 'type' => API_ID, 'in' => '0']
+			]],
 			'rights' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'replacement' => 'hostgroup_rights', 'uniq' => [['id']], 'fields' => [
 				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
 				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
@@ -262,11 +318,12 @@ class CUserGroup extends CApiService {
 
 		$this->checkDuplicates(array_column($usrgrps, 'name'));
 		$this->checkUsers($usrgrps);
-		$this->checkHimself($usrgrps, __FUNCTION__);
+		$this->checkOneself($usrgrps, __FUNCTION__);
 		$this->checkTemplateGroups($usrgrps);
 		$this->checkHostGroups($usrgrps);
 		$this->checkTagFilters($usrgrps);
 		self::checkUserDirectories($usrgrps);
+		self::checkMfaIds($usrgrps);
 	}
 
 	/**
@@ -294,7 +351,7 @@ class CUserGroup extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	private function validateUpdate(array &$usrgrps, array &$db_usrgrps = null) {
+	private function validateUpdate(array &$usrgrps, ?array &$db_usrgrps = null) {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE | API_ALLOW_UNEXPECTED, 'uniq' => [['usrgrpid']], 'fields' => [
 			'usrgrpid' =>	['type' => API_ID, 'flags' => API_REQUIRED]
 		]];
@@ -305,7 +362,7 @@ class CUserGroup extends CApiService {
 
 		$usrgrpids = array_column($usrgrps, 'usrgrpid');
 		$db_usrgrps = API::UserGroup()->get([
-			'output' => ['usrgrpid', 'name', 'debug_mode', 'gui_access', 'users_status'],
+			'output' => self::OUTPUT_FIELDS,
 			'usrgrpids' => $usrgrpids,
 			'preservekeys' => true
 		]);
@@ -322,7 +379,8 @@ class CUserGroup extends CApiService {
 		}
 
 		$names = [];
-		$usrgrps = $this->extendObjectsByKey($usrgrps, $db_usrgrps, 'usrgrpid', ['gui_access']);
+		$usrgrps = $this->extendObjectsByKey($usrgrps, $db_usrgrps, 'usrgrpid', ['gui_access', 'mfa_status']);
+
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
 			'usrgrpid' =>				['type' => API_ID],
 			'name' =>					['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
@@ -333,6 +391,11 @@ class CUserGroup extends CApiService {
 											['if' => ['field' => 'gui_access', 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_LDAP])], 'type' => API_ID],
 											['else' => true, 'type' => API_UNEXPECTED]
 			]],
+			'mfa_status' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_MFA_DISABLED, GROUP_MFA_ENABLED])],
+			'mfaid' =>					['type' => API_MULTIPLE, 'rules' => [
+											['if' => ['field' => 'mfa_status', 'in' => implode(',', [GROUP_MFA_ENABLED])], 'type' => API_ID],
+											['else' => true, 'type' => API_ID, 'in' => '0']
+			]],
 			'rights' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'replacement' => 'hostgroup_rights', 'uniq' => [['id']], 'fields' => [
 				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
 				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
@@ -389,13 +452,13 @@ class CUserGroup extends CApiService {
 		if ($names) {
 			$this->checkDuplicates($names);
 		}
-		$this->checkUsers($usrgrps);
-		$this->checkHimself($usrgrps, __FUNCTION__, $db_usrgrps);
-		$this->checkUsersWithoutGroups($usrgrps);
+		$this->checkUsers($usrgrps, $db_usrgrps);
+		$this->checkOneself($usrgrps, __FUNCTION__, $db_usrgrps);
 		$this->checkTemplateGroups($usrgrps);
 		$this->checkHostGroups($usrgrps);
 		$this->checkTagFilters($usrgrps);
 		self::checkUserDirectories($usrgrps);
+		self::checkMfaIds($usrgrps, $db_usrgrps);
 	}
 
 	/**
@@ -420,38 +483,55 @@ class CUserGroup extends CApiService {
 	/**
 	 * Check for valid users.
 	 *
-	 * @param array  $usrgrps
-	 * @param array  $usrgrps[]['users']              (optional)
-	 * @param string $usrgrps[]['users'][]['userid']
+	 * @param array      $user_groups
+	 * @param array|null $db_user_groups
 	 *
 	 * @throws APIException
 	 */
-	private function checkUsers(array $usrgrps) {
-		$userids = [];
+	private function checkUsers(array $user_groups, ?array &$db_user_groups = null) {
+		$user_indexes = [];
 
-		foreach ($usrgrps as $usrgrp) {
-			if (array_key_exists('users', $usrgrp)) {
-				foreach ($usrgrp['users'] as $user) {
-					$userids[$user['userid']] = true;
+		foreach ($user_groups as $i1 => $user_group) {
+			if (!array_key_exists('users', $user_group)) {
+				continue;
+			}
+
+			foreach ($user_group['users'] as $i2 => $user) {
+				$db_userids = $db_user_groups !== null
+					? array_column($db_user_groups[$user_group['usrgrpid']]['users'], 'userid')
+					: [];
+
+				if (!in_array($user['userid'], $db_userids)) {
+					$user_indexes[$user['userid']][$i1] = $i2;
 				}
 			}
 		}
 
-		if (!$userids) {
+		if (!$user_indexes) {
 			return;
 		}
 
-		$userids = array_keys($userids);
-
 		$db_users = DB::select('users', [
-			'output' => [],
-			'userids' => $userids,
+			'output' => ['userdirectoryid'],
+			'userids' => array_keys($user_indexes),
 			'preservekeys' => true
 		]);
 
-		foreach ($userids as $userid) {
+		foreach ($user_indexes as $userid => $indexes) {
 			if (!array_key_exists($userid, $db_users)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('User with ID "%1$s" is not available.', $userid));
+				$i1 = key($indexes);
+				$i2 = reset($indexes);
+
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
+					'/'.($i1 + 1).'/users/'.($i2 + 1).'/userid', _('object does not exist')
+				));
+			}
+
+			if ($db_users[$userid]['userdirectoryid'] != 0) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
+					'/'.($i1 + 1).'/users/'.($i2 + 1).'/userid',
+					_s('cannot update readonly parameter "%1$s" of provisioned user', 'usrgrps')
+				));
 			}
 		}
 	}
@@ -565,7 +645,7 @@ class CUserGroup extends CApiService {
 	}
 
 	/**
-	 * Auxiliary function for checkHimself().
+	 * Auxiliary function for checkOneself().
 	 * Returns true if user group has GROUP_GUI_ACCESS_DISABLED or GROUP_STATUS_DISABLED states.
 	 *
 	 * @param array  $usrgrp
@@ -574,7 +654,7 @@ class CUserGroup extends CApiService {
 	 *
 	 * @return bool
 	 */
-	private static function userGroupDisabled(array $usrgrp, $method, array $db_usrgrps = null) {
+	private static function userGroupDisabled(array $usrgrp, $method, ?array $db_usrgrps = null) {
 		$gui_access = array_key_exists('gui_access', $usrgrp)
 			? $usrgrp['gui_access']
 			: ($method === 'validateCreate' ? GROUP_GUI_ACCESS_SYSTEM : $db_usrgrps[$usrgrp['usrgrpid']]['gui_access']);
@@ -586,7 +666,7 @@ class CUserGroup extends CApiService {
 	}
 
 	/**
-	 * Additional check to exclude an opportunity to deactivate himself.
+	 * Additional check to exclude an opportunity to deactivate oneself.
 	 *
 	 * @param array  $usrgrps
 	 * @param string $method
@@ -594,7 +674,7 @@ class CUserGroup extends CApiService {
 	 *
 	 * @throws APIException
 	 */
-	private function checkHimself(array $usrgrps, $method, array $db_usrgrps = null) {
+	private function checkOneself(array $usrgrps, $method, ?array $db_usrgrps = null) {
 		if ($method === 'validateUpdate') {
 			$groups_users = [];
 
@@ -628,7 +708,7 @@ class CUserGroup extends CApiService {
 				foreach ($usrgrp['users'] as $user) {
 					if (bccomp(self::$userData['userid'], $user['userid']) == 0) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_('User cannot add himself to a disabled group or a group with disabled GUI access.')
+							_('User cannot add oneself to a disabled group or a group with disabled GUI access.')
 						);
 					}
 				}
@@ -637,88 +717,12 @@ class CUserGroup extends CApiService {
 	}
 
 	/**
-	 * Check to exclude an opportunity to leave user without user groups.
-	 *
-	 * @param array  $usrgrps
-	 * @param array  $usrgrps[]['usrgrpid']
-	 * @param array  $usrgrps[]['users']              (optional)
-	 * @param string $usrgrps[]['users'][]['userid']
-	 *
-	 * @throws APIException
-	 */
-	private function checkUsersWithoutGroups(array $usrgrps) {
-		$users_groups = [];
-
-		foreach ($usrgrps as $usrgrp) {
-			if (array_key_exists('users', $usrgrp)) {
-				$users_groups[$usrgrp['usrgrpid']] = [];
-
-				foreach ($usrgrp['users'] as $user) {
-					$users_groups[$usrgrp['usrgrpid']][$user['userid']] = true;
-				}
-			}
-		}
-
-		if (!$users_groups) {
-			return;
-		}
-
-		$db_users_groups = DB::select('users_groups', [
-			'output' => ['usrgrpid', 'userid'],
-			'filter' => ['usrgrpid' => array_keys($users_groups)]
-		]);
-
-		$ins_userids = [];
-		$del_userids = [];
-
-		foreach ($db_users_groups as $db_user_group) {
-			if (array_key_exists($db_user_group['userid'], $users_groups[$db_user_group['usrgrpid']])) {
-				unset($users_groups[$db_user_group['usrgrpid']][$db_user_group['userid']]);
-			}
-			else {
-				if (!array_key_exists($db_user_group['userid'], $del_userids)) {
-					$del_userids[$db_user_group['userid']] = 0;
-				}
-				$del_userids[$db_user_group['userid']]++;
-			}
-		}
-
-		foreach ($users_groups as $usrgrpid => $userids) {
-			foreach (array_keys($userids) as $userid) {
-				$ins_userids[$userid] = true;
-			}
-		}
-
-		$del_userids = array_diff_key($del_userids, $ins_userids);
-
-		if (!$del_userids) {
-			return;
-		}
-
-		$db_users = DBselect(
-			'SELECT u.userid,u.username,count(ug.usrgrpid) as usrgrp_num'.
-			' FROM users u,users_groups ug'.
-			' WHERE u.userid=ug.userid'.
-				' AND '.dbConditionInt('u.userid', array_keys($del_userids)).
-			' GROUP BY u.userid,u.username'
-		);
-
-		while ($db_user = DBfetch($db_users)) {
-			if ($db_user['usrgrp_num'] == $del_userids[$db_user['userid']]) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('User "%1$s" cannot be without user group.', $db_user['username'])
-				);
-			}
-		}
-	}
-
-	/**
-	 * @static
-	 *
 	 * @param array $usrgrps
 	 * @param array $db_usrgrps
 	 */
 	public static function updateForce($usrgrps, $db_usrgrps): void {
+		self::addFieldDefaultsByType($usrgrps, $db_usrgrps);
+
 		$upd_usrgrps = [];
 
 		foreach ($usrgrps as $usrgrp) {
@@ -745,14 +749,25 @@ class CUserGroup extends CApiService {
 		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_USER_GROUP, $usrgrps, $db_usrgrps);
 	}
 
+	private static function addFieldDefaultsByType(array &$usrgrps, array $db_usrgrps): void {
+		foreach ($usrgrps as &$usrgrp) {
+			if (array_key_exists('mfa_status', $usrgrp) && $usrgrp['mfa_status'] == GROUP_MFA_DISABLED
+					&& $db_usrgrps[$usrgrp['usrgrpid']]['mfaid'] != '0') {
+				$usrgrp['mfaid'] = '0';
+			}
+		}
+		unset($usrgrp);
+	}
+
 	/**
 	 * @param array      $usrgrps
 	 * @param null|array $db_usrgrps
 	 */
-	private static function updateRights(array &$usrgrps, array $db_usrgrps = null): void {
+	private static function updateRights(array &$usrgrps, ?array $db_usrgrps = null): void {
 		$ins_rights = [];
 		$upd_rights = [];
 		$del_rightids = [];
+		$changed_permissions = [];
 
 		foreach (['hostgroup_rights', 'templategroup_rights'] as $parameter) {
 			foreach ($usrgrps as &$usrgrp) {
@@ -778,6 +793,8 @@ class CUserGroup extends CApiService {
 								'values' => $upd_right,
 								'where' => ['rightid' => $db_right['rightid']]
 							];
+							$changed_permissions[$usrgrp['usrgrpid']][$right['id']]['new'] = $right['permission'];
+							$changed_permissions[$usrgrp['usrgrpid']][$right['id']]['old'] = $db_right['permission'];
 						}
 					}
 					else {
@@ -786,15 +803,34 @@ class CUserGroup extends CApiService {
 							'id' => $right['id'],
 							'permission' => $right['permission']
 						];
+
+						if ($db_usrgrps !== null) {
+							$changed_permissions[$usrgrp['usrgrpid']][$right['id']]['new'] = $right['permission'];
+							$changed_permissions[$usrgrp['usrgrpid']][$right['id']]['old'] = PERM_NONE;
+						}
 					}
 				}
 				unset($right);
 
-				$del_rightids = array_merge($del_rightids, array_column($db_rights, 'rightid'));
+				foreach ($db_rights as $db_right) {
+					$del_rightids[] = $db_right['rightid'];
+					$changed_permissions[$usrgrp['usrgrpid']][$db_right['id']]['new'] = PERM_NONE;
+					$changed_permissions[$usrgrp['usrgrpid']][$db_right['id']]['old'] = $db_right['permission'];
+				}
 			}
 			unset($usrgrp);
 		}
 
+		if ($changed_permissions) {
+			$ugset_permissions = self::getUgSetPermissions($changed_permissions);
+
+			if ($ugset_permissions) {
+				$db_ugset_permissions = self::getDbUgSetPermissions($ugset_permissions);
+
+				self::updatePermissions($ugset_permissions, $db_ugset_permissions);
+			}
+		}
+
 		if ($ins_rights) {
 			$rightids = DB::insertBatch('rights', $ins_rights);
 		}
@@ -824,11 +860,239 @@ class CUserGroup extends CApiService {
 		}
 	}
 
+	private static function getUgSetPermissions(array $changed_permissions): array {
+		$ugset_permissions = [];
+
+		$options = [
+			'output' => ['usrgrpid', 'ugsetid'],
+			'filter' => [
+				'usrgrpid' => array_keys($changed_permissions)
+			]
+		];
+		$result = DBselect(DB::makeSql('ugset_group', $options));
+
+		while ($row = DBfetch($result)) {
+			foreach ($changed_permissions[$row['usrgrpid']] as $groupid => $permission) {
+				if (!array_key_exists($row['ugsetid'], $ugset_permissions)
+						|| !array_key_exists($groupid, $ugset_permissions[$row['ugsetid']])) {
+					$ugset_permissions[$row['ugsetid']][$groupid]['new'] = $permission['new'];
+					$ugset_permissions[$row['ugsetid']][$groupid]['old'] = $permission['old'];
+					$ugset_permissions[$row['ugsetid']][$groupid]['usrgrp_count'] = 1;
+				}
+				else {
+					if ($ugset_permissions[$row['ugsetid']][$groupid]['new'] != PERM_DENY
+							&& ($permission['new'] > $ugset_permissions[$row['ugsetid']][$groupid]['new']
+								|| $permission['new'] == PERM_DENY)) {
+						$ugset_permissions[$row['ugsetid']][$groupid]['new'] = $permission['new'];
+					}
+
+					if ($permission['old'] == $ugset_permissions[$row['ugsetid']][$groupid]['old']) {
+						$ugset_permissions[$row['ugsetid']][$groupid]['usrgrp_count']++;
+					}
+					elseif ($ugset_permissions[$row['ugsetid']][$groupid]['old'] != PERM_DENY
+							&& ($permission['old'] > $ugset_permissions[$row['ugsetid']][$groupid]['old']
+								|| $permission['old'] == PERM_DENY)) {
+						$ugset_permissions[$row['ugsetid']][$groupid]['old'] = $permission['old'];
+						$ugset_permissions[$row['ugsetid']][$groupid]['usrgrp_count'] = 1;
+					}
+				}
+			}
+		}
+
+		return $ugset_permissions;
+	}
+
+	private static function getDbUgSetPermissions(array $ugset_permissions): array {
+		$db_ugset_permissions = array_fill_keys(array_keys($ugset_permissions), []);
+
+		$result = DBselect(
+			'SELECT ugg.ugsetid,ugg.usrgrpid,r.id,r.permission'.
+			' FROM ugset_group ugg'.
+			' JOIN rights r ON ugg.usrgrpid=r.groupid'.
+			' WHERE '.dbConditionId('ugg.ugsetid', array_keys($ugset_permissions))
+		);
+
+		while ($row = DBfetch($result)) {
+			if (!array_key_exists($row['ugsetid'], $db_ugset_permissions)
+					|| !array_key_exists($row['id'], $db_ugset_permissions[$row['ugsetid']])) {
+				$db_ugset_permissions[$row['ugsetid']][$row['id']]['old'] = $row['permission'];
+				$db_ugset_permissions[$row['ugsetid']][$row['id']]['usrgrp_count'] = 1;
+			}
+			else {
+				if ($row['permission'] == $db_ugset_permissions[$row['ugsetid']][$row['id']]['old']) {
+					$db_ugset_permissions[$row['ugsetid']][$row['id']]['usrgrp_count']++;
+				}
+				elseif ($db_ugset_permissions[$row['ugsetid']][$row['id']]['old'] != PERM_DENY
+					&& ($row['permission'] > $db_ugset_permissions[$row['ugsetid']][$row['id']]['old']
+						|| $row['permission'] == PERM_DENY)) {
+					$db_ugset_permissions[$row['ugsetid']][$row['id']]['old'] = $row['permission'];
+					$db_ugset_permissions[$row['ugsetid']][$row['id']]['usrgrp_count'] = 1;
+				}
+			}
+		}
+
+		return $db_ugset_permissions;
+	}
+
+	private static function updatePermissions(array $ugset_permissions, array $db_ugset_permissions): void {
+		$ins_permissions = [];
+		$upd_permissions = [];
+		$del_permissions = [];
+
+		self::unsetUnhangedUgSetPermissions($ugset_permissions, $db_ugset_permissions);
+
+		if (!$ugset_permissions) {
+			return;
+		}
+
+		[$permissions, $db_permissions] = self::getPermissions($ugset_permissions, $db_ugset_permissions);
+
+		if (!$permissions) {
+			return;
+		}
+
+		foreach ($permissions as $ugsetid => $hgset_permissions) {
+			foreach ($hgset_permissions as $hgsetid => $permission) {
+				$db_permission = $db_permissions[$ugsetid][$hgsetid];
+
+				if ($permission >= PERM_READ) {
+					if ($db_permission >= PERM_READ) {
+						if ($permission != $db_permission) {
+							$upd_permissions[] = [
+								'values' => ['permission' => $permission],
+								'where' => ['ugsetid' => $ugsetid, 'hgsetid' => $hgsetid]
+							];
+						}
+					}
+					else {
+						$ins_permissions[] = [
+							'ugsetid' => $ugsetid,
+							'hgsetid' => $hgsetid,
+							'permission' => $permission
+						];
+					}
+				}
+				elseif ($db_permission >= PERM_READ) {
+					$del_permissions[] = dbConditionId('ugsetid', [$ugsetid]).
+						' AND '.dbConditionId('hgsetid', [$hgsetid]);
+				}
+			}
+		}
+
+		if ($del_permissions) {
+			DBexecute('DELETE FROM permission WHERE ('.implode(') OR (', $del_permissions).')');
+		}
+
+		if ($upd_permissions) {
+			DB::update('permission', $upd_permissions);
+		}
+
+		if ($ins_permissions) {
+			DB::insert('permission', $ins_permissions, false);
+		}
+	}
+
+	private static function unsetUnhangedUgSetPermissions(array &$ugset_permissions,
+			array &$db_ugset_permissions): void {
+		foreach ($ugset_permissions as $ugsetid => $group_permissions) {
+			foreach ($group_permissions as $groupid => $permission) {
+				$changed = false;
+
+				if (!array_key_exists($groupid, $db_ugset_permissions[$ugsetid])) {
+					$changed = true;
+				}
+				else {
+					$db_permission = $db_ugset_permissions[$ugsetid][$groupid];
+
+					if ($permission['new'] > $db_permission['old']
+							|| ($permission['new'] == PERM_DENY && $db_permission['old'] != PERM_DENY)
+							|| ($permission['old'] == $db_permission['old']
+								&& $permission['usrgrp_count'] == $db_permission['usrgrp_count'])) {
+						$changed = true;
+					}
+				}
+
+				if (!$changed) {
+					unset($ugset_permissions[$ugsetid][$groupid]);
+				}
+			}
+
+			if (!$ugset_permissions[$ugsetid]) {
+				unset($ugset_permissions[$ugsetid], $db_ugset_permissions[$ugsetid]);
+			}
+		}
+	}
+
+	private static function getPermissions(array $ugset_permissions, array $db_ugset_permissions): array {
+		$permissions = [];
+		$db_permissions = [];
+
+		$groupids = [];
+		$group_ugsetids = [];
+
+		foreach ($ugset_permissions as $ugsetid => $group_permissions) {
+			$groupids += $group_permissions;
+
+			foreach ($group_permissions as $groupid => $foo) {
+				$group_ugsetids[$groupid][$ugsetid] = true;
+			}
+
+			foreach ($db_ugset_permissions[$ugsetid] as $groupid => $foo) {
+				$group_ugsetids[$groupid][$ugsetid] = true;
+			}
+		}
+
+		$result = DBselect(
+			'SELECT hgg.hgsetid,hgg.groupid'.
+			' FROM hgset_group hgg'.
+			' WHERE hgg.hgsetid IN('.
+					'SELECT DISTINCT hgg1.hgsetid'.
+					' FROM hgset_group hgg1'.
+					' WHERE '.dbConditionId('hgg1.groupid', array_keys($groupids)).
+				')'.
+				' AND '.dbConditionId('hgg.groupid', array_keys($group_ugsetids))
+		);
+
+		while ($row = DBfetch($result)) {
+			foreach ($group_ugsetids[$row['groupid']] as $ugsetid => $foo) {
+				if (!array_key_exists($row['groupid'], $ugset_permissions[$ugsetid])
+						&& !array_key_exists($row['groupid'], $db_ugset_permissions[$ugsetid])) {
+					continue;
+				}
+
+				$permission = array_key_exists($row['groupid'], $ugset_permissions[$ugsetid])
+					? $ugset_permissions[$ugsetid][$row['groupid']]['new']
+					: $db_ugset_permissions[$ugsetid][$row['groupid']]['old'];
+
+				if (!array_key_exists($ugsetid, $permissions)
+						|| !array_key_exists($row['hgsetid'], $permissions[$ugsetid])
+						|| ($permissions[$ugsetid][$row['hgsetid']] != PERM_DENY
+							&& ($permission > $permissions[$ugsetid][$row['hgsetid']] || $permission == PERM_DENY))) {
+					$permissions[$ugsetid][$row['hgsetid']] = $permission;
+				}
+
+				$db_permission = array_key_exists($row['groupid'], $db_ugset_permissions[$ugsetid])
+					? $db_ugset_permissions[$ugsetid][$row['groupid']]['old']
+					: PERM_NONE;
+
+				if (!array_key_exists($ugsetid, $db_permissions)
+						|| !array_key_exists($row['hgsetid'], $db_permissions[$ugsetid])
+						|| ($db_permissions[$ugsetid][$row['hgsetid']] != PERM_DENY
+							&& ($db_permission > $db_permissions[$ugsetid][$row['hgsetid']]
+								|| $db_permission == PERM_DENY))) {
+					$db_permissions[$ugsetid][$row['hgsetid']] = $db_permission;
+				}
+			}
+		}
+
+		return [$permissions, $db_permissions];
+	}
+
 	/**
 	 * @param array      $usrgrps
 	 * @param null|array $db_usrgrps
 	 */
-	private static function updateTagFilters(array &$usrgrps, array $db_usrgrps = null): void {
+	private static function updateTagFilters(array &$usrgrps, ?array $db_usrgrps = null): void {
 		$ins_tag_filters = [];
 		$del_tag_filterids = [];
 
@@ -903,64 +1167,56 @@ class CUserGroup extends CApiService {
 	 * @param array      $groups
 	 * @param null|array $db_groups
 	 */
-	private static function updateUsers(array &$groups, array $db_groups = null): void {
+	private static function updateUsers(array &$groups, ?array &$db_groups = null): void {
 		$users = [];
-		$db_users = [];
+		$del_user_usrgrpids = [];
 
 		foreach ($groups as &$group) {
 			if (!array_key_exists('users', $group)) {
 				continue;
 			}
 
-			$_db_users = $db_groups !== null
+			$userids = array_column($group['users'], 'userid');
+			$db_users = $db_groups !== null
 				? array_column($db_groups[$group['usrgrpid']]['users'], null, 'userid')
 				: [];
 
-			foreach ($group['users'] as $user) {
-				$userid = $user['userid'];
+			$del_userids = array_diff(array_keys($db_users), $userids);
 
-				if (array_key_exists($userid, $_db_users)) {
-					unset($_db_users[$userid]);
+			if (!array_diff($userids, array_keys($db_users)) && !$del_userids) {
+				unset($group['users'], $db_groups[$group['usrgrpid']]['users']);
+				continue;
+			}
+
+			foreach ($del_userids as $del_userid) {
+				if ($db_users[$del_userid]['userdirectoryid'] != 0) {
+					unset($db_groups[$group['usrgrpid']]['users'][$db_users[$del_userid]['id']]);
 				}
 				else {
-					$users[$userid]['userid'] = $userid;
-					$users[$userid]['usrgrps'][] = ['usrgrpid' => $group['usrgrpid']];
-
-					$db_users[$userid]['userid'] = $userid;
-					$db_users[$userid]['usrgrps'] = [];
+					$del_user_usrgrpids[$del_userid][] = $group['usrgrpid'];
 				}
 			}
 
-			foreach ($_db_users as $userid => $db_user) {
-				$users[$userid]['userid'] = $userid;
-				$users[$userid] += ['usrgrps' => []];
+			foreach ($group['users'] as $user) {
+				$users[$user['userid']]['userid'] = $user['userid'];
+				$users[$user['userid']]['usrgrps'][]['usrgrpid'] = $group['usrgrpid'];
+			}
 
-				$db_users[$userid]['userid'] = $userid;
-				$db_users[$userid]['usrgrps'][$db_user['id']] = [
-					'id' => $db_user['id'],
-					'usrgrpid' => $group['usrgrpid']
-				];
+			if ($db_groups !== null) {
+				foreach ($db_groups[$group['usrgrpid']]['users'] as $db_user) {
+					if (!array_key_exists($db_user['userid'], $users)) {
+						$users[$db_user['userid']]['userid'] = $db_user['userid'];
+						$users[$db_user['userid']]['usrgrps'] = [];
+					}
+				}
 			}
 
 			unset($group['users'], $db_groups[$group['usrgrpid']]['users']);
 		}
-		unset($group);
 
-		if (!$users) {
-			return;
+		if ($users) {
+			CUser::updateFromUserGroup($users, $del_user_usrgrpids);
 		}
-
-		$options = [
-			'output' => ['userid', 'username'],
-			'userids' => array_keys($users)
-		];
-		$result = DBselect(DB::makeSql('users', $options));
-
-		while ($row = DBfetch($result)) {
-			$db_users[$row['userid']]['username'] = $row['username'];
-		}
-
-		CUser::updateForce(array_values($users), $db_users);
 	}
 
 	/**
@@ -977,8 +1233,9 @@ class CUserGroup extends CApiService {
 
 		$this->validateDelete($usrgrpids, $db_usrgrps);
 
+		self::unlinkUsers($db_usrgrps);
+
 		DB::delete('rights', ['groupid' => $usrgrpids]);
-		DB::delete('users_groups', ['usrgrpid' => $usrgrpids]);
 		DB::delete('usrgrp', ['usrgrpid' => $usrgrpids]);
 
 		self::addAuditLog(CAudit::ACTION_DELETE, CAudit::RESOURCE_USER_GROUP, $db_usrgrps);
@@ -992,7 +1249,7 @@ class CUserGroup extends CApiService {
 	 * @param array $usrgrpids
 	 * @param array $db_usrgrps
 	 */
-	protected function validateDelete(array &$usrgrpids, array &$db_usrgrps = null) {
+	protected function validateDelete(array &$usrgrpids, ?array &$db_usrgrps = null) {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 		if (!CApiInputValidator::validate($api_input_rules, $usrgrpids, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
@@ -1075,7 +1332,69 @@ class CUserGroup extends CApiService {
 			));
 		}
 
-		$this->checkUsersWithoutGroups($usrgrps);
+		self::checkProvisionedUsersExist($db_usrgrps);
+		self::checkUsedInProvisionGroupMapping($db_usrgrps);
+	}
+
+	private static function checkProvisionedUsersExist(array $db_user_groups): void {
+		$row = DBfetch(DBselect(
+			'SELECT ug.usrgrpid,u.username'.
+			' FROM users_groups ug,users u'.
+			' WHERE ug.userid=u.userid'.
+				' AND u.userdirectoryid IS NOT NULL'.
+				' AND '.dbConditionId('ug.usrgrpid', array_keys($db_user_groups)),
+			1
+		));
+
+		if ($row) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Cannot delete user group "%1$s", because it is used by provisioned user "%2$s".',
+					$db_user_groups[$row['usrgrpid']]['name'], $row['username']
+				)
+			);
+		}
+	}
+
+	private static function checkUsedInProvisionGroupMapping(array $db_usrgrps): void {
+		$row = DBfetch(DBselect(
+			'SELECT ud.name,ud.idp_type,udug.usrgrpid'.
+			' FROM userdirectory_usrgrp udug'.
+			' JOIN userdirectory_idpgroup udig ON udug.userdirectory_idpgroupid=udig.userdirectory_idpgroupid'.
+			' JOIN userdirectory ud ON udig.userdirectoryid=ud.userdirectoryid'.
+			' WHERE '.dbConditionId('udug.usrgrpid', array_keys($db_usrgrps)),
+			1
+		));
+
+		if (!$row) {
+			return;
+		}
+
+		if ($row['idp_type'] == IDP_TYPE_SAML) {
+			$error = _s('Cannot delete user group "%1$s", because it is used by SAML userdirectory.',
+				$db_usrgrps[$row['usrgrpid']]['name']
+			);
+		}
+		else {
+			$error = _s('Cannot delete user group "%1$s", because it is used by LDAP userdirectory "%2$s".',
+				$db_usrgrps[$row['usrgrpid']]['name'], $row['name']
+			);
+		}
+
+		self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+	}
+
+	private static function unlinkUsers(array $db_groups): void {
+		$groups = [];
+
+		foreach ($db_groups as $db_group) {
+			$groups[] = [
+				'usrgrpid' => $db_group['usrgrpid'],
+				'users' => []
+			];
+		}
+
+		self::addAffectedObjects($groups, $db_groups);
+		self::updateUsers($groups, $db_groups);
 	}
 
 	protected function addRelatedObjects(array $options, array $result) {
@@ -1215,8 +1534,6 @@ class CUserGroup extends CApiService {
 	/**
 	 * Add the existing rights, tag_filters and userids to $db_usrgrps whether these are affected by the update.
 	 *
-	 * @static
-	 *
 	 * @param array $usrgrps
 	 * @param array $db_usrgrps
 	 */
@@ -1289,11 +1606,12 @@ class CUserGroup extends CApiService {
 		}
 
 		if ($usrgrpids['users']) {
-			$options = [
-				'output' => ['id', 'usrgrpid', 'userid'],
-				'filter' => ['usrgrpid' => $usrgrpids['users']]
-			];
-			$db_users = DBselect(DB::makeSql('users_groups', $options));
+			$db_users = DBselect(
+				'SELECT ug.id,ug.usrgrpid,ug.userid,u.userdirectoryid'.
+				' FROM users_groups ug,users u'.
+				' WHERE ug.userid=u.userid'.
+					' AND '.dbConditionId('ug.usrgrpid', $usrgrpids['users'])
+			);
 
 			while ($db_user = DBfetch($db_users)) {
 				$db_usrgrps[$db_user['usrgrpid']]['users'][$db_user['id']] =
@@ -1335,4 +1653,40 @@ class CUserGroup extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Check for valid MFA method.
+	 *
+	 * @param array      $user_groups
+	 * @param array|null $db_user_groups
+	 *
+	 * @throws APIException
+	 */
+	private static function checkMfaIds(array $user_groups, ?array $db_user_groups = null): void {
+		foreach ($user_groups as $i => $user_group) {
+			if (!array_key_exists('mfaid', $user_group) || $user_group['mfaid'] == 0
+					|| ($db_user_groups !== null
+						&& bccomp($user_group['mfaid'], $db_user_groups[$user_group['usrgrpid']]['mfaid']) == 0)) {
+				unset($user_groups[$i]);
+			}
+		}
+
+		if (!$user_groups) {
+			return;
+		}
+
+		$db_mfas = DB::select('mfa', [
+			'output' => [],
+			'mfaids' => array_column($user_groups, 'mfaid'),
+			'preservekeys' => true
+		]);
+
+		foreach ($user_groups as $i => $user_group) {
+			if (!array_key_exists($user_group['mfaid'], $db_mfas)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1).'/mfaid', _('object does not exist'))
+				);
+			}
+		}
+	}
 }
