@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -172,7 +172,12 @@ class MysqlDbBackend extends DbBackend {
 	 * @return bool
 	 */
 	public function init() {
-		$db_encoding = DBselect('SHOW VARIABLES LIKE "character_set_database"');
+		$db_innodb_snapshot_isolation = DBfetch(DBselect("SHOW VARIABLES LIKE 'innodb_snapshot_isolation'"));
+		if ($db_innodb_snapshot_isolation && $db_innodb_snapshot_isolation['Value'] !== 'OFF') {
+			DBexecute("SET innodb_snapshot_isolation='OFF'");
+		}
+
+		$db_encoding = DBselect("SHOW VARIABLES LIKE 'character_set_database'");
 		$charset = $db_encoding ? DBfetch($db_encoding) : false;
 		if ($charset && strtoupper($charset['Value']) === 'UTF8MB4') {
 			DBexecute('SET NAMES utf8mb4');
