@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -147,10 +142,12 @@ class CZabbixServer {
 	 * @param string      $sid
 	 * @param null|string $hostid
 	 * @param null|string $eventid
+	 * @param null|string $manualinput
 	 *
 	 * @return bool|array
 	 */
-	public function executeScript(string $scriptid, string $sid, ?string $hostid = null, ?string $eventid = null) {
+	public function executeScript(string $scriptid, string $sid, ?string $hostid = null, ?string $eventid = null,
+			$manualinput = null) {
 		$params = [
 			'request' => 'command',
 			'scriptid' => $scriptid,
@@ -166,40 +163,42 @@ class CZabbixServer {
 			$params['eventid'] = $eventid;
 		}
 
+		if ($manualinput !== null) {
+			$params['manualinput'] = $manualinput;
+		}
+
 		return $this->request($params);
 	}
 
 	/**
-	 * Request server to test item preprocessing steps.
+	 * @param array  $data
+	 *        string $data['itemid']  (optional) Item ID.
+	 *        string $data['host']    (optional) Technical name of the host.
+	 *        string $data['key']     (optional) Item key.
+	 *        string $data['value']   Item value.
+	 *        string $data['clock']   (optional) Time when the value was received.
+	 *        string $data['ns']      (optional) Nanoseconds when the value was received.
+	 * @param string $sid             User session ID or user API token.
 	 *
-	 * @param array  $data                                     Array of preprocessing steps test.
-	 * @param string $data['value']                            Value to use for preprocessing step testing.
-	 * @param int    $data['value_type']                       Item value type.
-	 * @param array  $data['history']                          Previous value object.
-	 * @param string $data['history']['value']                 Previous value.
-	 * @param string $data['history']['timestamp']             Previous value time.
-	 * @param array  $data['steps']                            Preprocessing step object.
-	 * @param int    $data['steps'][]['type']                  Type of preprocessing step.
-	 * @param string $data['steps'][]['params']                Parameters of preprocessing step.
-	 * @param int    $data['steps'][]['error_handler']         Error handler selected as "custom on fail".
-	 * @param string $data['steps'][]['error_handler_params']  Parameters configured for selected error handler.
-	 * @param string $sid                                      User session ID.
-	 *
-	 * @return array
+	 * @return array|bool
 	 */
-	public function testPreprocessingSteps(array $data, $sid) {
+	public function pushHistory(array $data, string $sid) {
 		return $this->request([
-			'request' => 'preprocessing.test',
+			'request' => 'history.push',
 			'data' => $data,
-			'sid' => $sid
+			'sid' => $sid,
+			'clientip' => CWebUser::getIp()
 		]);
 	}
 
 	/**
 	 * Request server to test item.
 	 *
-	 * @param array  $data  Array of item properties to test.
-	 * @param string $sid   User session ID.
+	 * @param array  $data
+	 * @param array  $data['item']     Array of item parameters.
+	 * @param array  $data['host']     (optional) Array of host parameters.
+	 * @param array  $data['options']  (optional) Array of test parameters.
+	 * @param string $sid              User session ID.
 	 *
 	 * @return array|bool
 	 */
@@ -344,6 +343,9 @@ class CZabbixServer {
 					'proxyid' =>				['type' => API_ID, 'flags' => API_REQUIRED]
 				]],
 				'count' =>					['type' => API_STRING_UTF8, 'flags' => API_REQUIRED]	// API_FLOAT 0-n
+			]],
+			'server stats' =>			['type' => API_OBJECT, 'flags' => API_REQUIRED, 'fields' => [
+				'version' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED]
 			]]
 		]];
 
@@ -476,6 +478,9 @@ class CZabbixServer {
 		$json = json_encode($params);
 		if (fwrite($this->socket, ZBX_TCP_HEADER.pack('V', strlen($json))."\x00\x00\x00\x00".$json) === false) {
 			$this->error = _s('Cannot send command, check connection with Zabbix server "%1$s".', $this->host);
+
+			fclose($this->socket);
+
 			return false;
 		}
 
@@ -483,59 +488,65 @@ class CZabbixServer {
 		$response = '';
 		$response_len = 0;
 		$expected_len = null;
-		$now = time();
 
-		while (true) {
-			if ((time() - $now) >= $this->timeout) {
-				$this->error = _s(
-					'Connection timeout of %1$s seconds exceeded when connecting to Zabbix server "%2$s".',
-					$this->timeout, $this->host
-				);
+		while (!feof($this->socket)) {
+			if (($buffer = fread($this->socket, self::READ_BYTES_LIMIT)) === false) {
+				$info = stream_get_meta_data($this->socket);
+
+				if ($info['timed_out']) {
+					$this->error = _s(
+						'Response timeout of %1$s exceeded when connecting to Zabbix server "%2$s".',
+						secondsToPeriod($this->timeout), $this->host
+					);
+				} else {
+					$this->error = _s('Cannot read response from Zabbix server "%1$s".', $this->host);
+				}
+
+				fclose($this->socket);
+
 				return false;
 			}
 
-			if (!feof($this->socket) && ($buffer = fread($this->socket, self::READ_BYTES_LIMIT)) !== false) {
-				$response_len += strlen($buffer);
-				$response .= $buffer;
+			$response_len += strlen($buffer);
+			$response .= $buffer;
 
-				if ($expect == self::ZBX_TCP_EXPECT_HEADER) {
-					if (strncmp($response, ZBX_TCP_HEADER, min($response_len, ZBX_TCP_HEADER_LEN)) != 0) {
-						$this->error = _s('Incorrect response received from Zabbix server "%1$s".', $this->host);
-						return false;
-					}
+			if ($expect == self::ZBX_TCP_EXPECT_HEADER) {
+				if (strncmp($response, ZBX_TCP_HEADER, min($response_len, ZBX_TCP_HEADER_LEN)) != 0) {
+					$this->error = _s('Incorrect response received from Zabbix server "%1$s".', $this->host);
 
-					if ($response_len < ZBX_TCP_HEADER_LEN) {
-						continue;
-					}
+					fclose($this->socket);
 
-					$expect = self::ZBX_TCP_EXPECT_DATA;
+					return false;
 				}
 
-				if ($response_len < ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN) {
+				if ($response_len < ZBX_TCP_HEADER_LEN) {
 					continue;
 				}
 
-				if ($expected_len === null) {
-					$expected_len = unpack('Vlen', substr($response, ZBX_TCP_HEADER_LEN, 4))['len'];
-					$expected_len += ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN;
+				$expect = self::ZBX_TCP_EXPECT_DATA;
+			}
 
-					if ($this->totalBytesLimit != 0 && $expected_len >= $this->totalBytesLimit) {
-						$this->error = _s(
-							'Size of the response received from Zabbix server "%1$s" exceeds the allowed size of %2$s bytes. This value can be increased in the ZBX_SOCKET_BYTES_LIMIT constant in include/defines.inc.php.',
-							$this->host, $this->totalBytesLimit
-						);
-						return false;
-					}
-				}
+			if ($response_len < ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN) {
+				continue;
+			}
 
-				if ($response_len >= $expected_len) {
-					break;
+			if ($expected_len === null) {
+				$expected_len = unpack('Vlen', substr($response, ZBX_TCP_HEADER_LEN, 4))['len'];
+				$expected_len += ZBX_TCP_HEADER_LEN + ZBX_TCP_DATALEN_LEN;
+
+				if ($this->totalBytesLimit != 0 && $expected_len >= $this->totalBytesLimit) {
+					$this->error = _s(
+						'Size of the response received from Zabbix server "%1$s" exceeds the allowed size of %2$s bytes. This value can be increased in the ZBX_SOCKET_BYTES_LIMIT constant in include/defines.inc.php.',
+						$this->host, $this->totalBytesLimit
+					);
+					fclose($this->socket);
+
+					return false;
 				}
 			}
-			else {
-				$this->error =
-					_s('Cannot read the response, check connection with the Zabbix server "%1$s".', $this->host);
-				return false;
+
+			if ($response_len >= $expected_len) {
+				break;
 			}
 		}
 
