@@ -1,49 +1,43 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 namespace Zabbix\Widgets\Fields;
 
-use CArrayHelper,
-	CNumberParser,
+use CNumberParser,
 	CParser;
 
 use Zabbix\Widgets\CWidgetField;
 
 class CWidgetFieldThresholds extends CWidgetField {
 
+	public const DEFAULT_VIEW = \CWidgetFieldThresholdsView::class;
 	public const DEFAULT_VALUE = [];
 
-	private bool $is_binary_units = false;
+	private bool $is_binary_units;
 
 	/**
 	 * Create widget field for Thresholds selection.
 	 */
-	public function __construct(string $name, string $label = null, bool $is_binary_units = false) {
+	public function __construct(string $name, ?string $label = null, bool $is_binary_units = false) {
 		parent::__construct($name, $label);
 
 		$this->is_binary_units = $is_binary_units;
 
 		$this
 			->setDefault(self::DEFAULT_VALUE)
-			->setSaveType(ZBX_WIDGET_FIELD_TYPE_STR)
 			->setValidationRules(['type' =>  API_OBJECTS, 'uniq' => [['threshold']], 'fields' => [
 				'color'		=> ['type' => API_COLOR, 'flags' => API_REQUIRED | API_NOT_EMPTY],
 				'threshold'	=> ['type' => API_NUMERIC, 'flags' => API_REQUIRED]
@@ -65,9 +59,7 @@ class CWidgetFieldThresholds extends CWidgetField {
 	}
 
 	public function validate($strict = false): array {
-		$errors = parent::validate($strict);
-
-		if ($errors) {
+		if ($errors = parent::validate($strict)) {
 			return $errors;
 		}
 
@@ -80,7 +72,7 @@ class CWidgetFieldThresholds extends CWidgetField {
 		$thresholds = [];
 
 		foreach ($this->getValue() as $threshold) {
-			if ($number_parser->parse($threshold['threshold']) == CParser::PARSE_SUCCESS) {
+			if ($number_parser->parse($threshold['threshold']) === CParser::PARSE_SUCCESS) {
 				$thresholds[] = $threshold + ['threshold_value' => $number_parser->calcValue()];
 			}
 		}
@@ -104,18 +96,16 @@ class CWidgetFieldThresholds extends CWidgetField {
 	}
 
 	public function toApi(array &$widget_fields = []): void {
-		$value = $this->getValue();
-
-		foreach ($value as $index => $val) {
+		foreach ($this->getValue() as $index => $value) {
 			$widget_fields[] = [
 				'type' => $this->save_type,
-				'name' => $this->name.'.color.'.$index,
-				'value' => $val['color']
+				'name' => $this->name.'.'.$index.'.color',
+				'value' => $value['color']
 			];
 			$widget_fields[] = [
 				'type' => $this->save_type,
-				'name' => $this->name.'.threshold.'.$index,
-				'value' => $val['threshold']
+				'name' => $this->name.'.'.$index.'.threshold',
+				'value' => $value['threshold']
 			];
 		}
 	}
