@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -1203,19 +1203,6 @@ function renderInterfaceHeaders() {
 		);
 }
 
-function getHostDashboards(string $hostid, array $dashboard_fields = []): array {
-	$dashboard_fields = array_merge($dashboard_fields, ['dashboardid']);
-	$dashboard_fields = array_keys(array_flip($dashboard_fields));
-
-	$templateids = CApiHostHelper::getParentTemplates([$hostid])[1];
-
-	return API::TemplateDashboard()->get([
-		'output' => $dashboard_fields,
-		'templateids' => $templateids,
-		'preservekeys' => true
-	]);
-}
-
 /**
  * Return macro value to display in the list of inherited macros.
  *
@@ -1414,11 +1401,17 @@ function getSanitizedHostPrototypeInterfaceDetailsFields(array $details): array
 /**
  * Get summary interface availability status.
  *
- * @param array  $interfaces
+ * @param array $interfaces
  *
  * @return int
  */
 function getInterfaceAvailabilityStatus(array $interfaces): int {
+	$interfaces_with_enabled_items = array_filter($interfaces,
+		static fn ($interface) => $interface['has_enabled_items']
+	);
+
+	$interfaces = $interfaces_with_enabled_items ?: $interfaces;
+
 	$available = array_column($interfaces, 'available');
 
 	if (in_array(INTERFACE_AVAILABLE_MIXED, $available)) {
