@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -57,16 +52,24 @@
 				? <?= json_encode(_('Key passphrase')) ?>
 				: <?= json_encode(_('Password')) ?>;
 		}
+		else {
+			document.getElementById('js-item-password-label').innerText = <?= json_encode(_('Password')) ?>;
+		}
 	}
 
 	const item_form = {
 		init({interfaces, value_type_by_keys, keys_by_item_type, testable_item_types, field_switches, interface_types,
-				discovered_item}) {
+				discovered_item, inherited_timeouts}) {
 			this.interfaces = interfaces;
 			this.testable_item_types = testable_item_types;
 			this.field_switches = field_switches;
 			this.interface_types = interface_types;
 			this.discovered_item = discovered_item === undefined ? false : discovered_item;
+			this.type = document.getElementById('type');
+			this.custom_timeout = document.getElementById('custom_timeout');
+			this.inherited_timeout = document.getElementById('inherited_timeout');
+			this.timeout = document.getElementById('timeout');
+			this.inherited_timeouts = inherited_timeouts;
 
 			if (typeof value_type_by_keys !== 'undefined' && typeof keys_by_item_type !== 'undefined') {
 				item_type_lookup.init(value_type_by_keys, keys_by_item_type, this.discovered_item);
@@ -83,15 +86,27 @@
 					array_keys(itemTypeInterface(), INTERFACE_TYPE_OPT)
 				) ?>.indexOf(type) != -1;
 
-		if (type == <?= ITEM_TYPE_SIMPLE ?> && (key.substr(0, 7) === 'vmware.' || key.substr(0, 8) === 'icmpping')) {
+		if (type == <?= ITEM_TYPE_SIMPLE ?>
+				&& (key.substring(0, 7) === 'vmware.' || key.substring(0, 8) === 'icmpping')) {
 			jQuery('#test_item').prop('disabled', true);
 		}
 		else {
 			jQuery('#test_item').prop('disabled', (testable_item_types.indexOf(type) == -1));
 		}
 
-		// delay field
-		if (type == <?= ITEM_TYPE_ZABBIX_ACTIVE ?>) {
+		if (type == <?= ITEM_TYPE_SIMPLE ?>) {
+			const toggle_fields = [
+				'js-item-timeout-label',
+				'js-item-timeout-field'
+			];
+			const set_hidden = key.substring(0, 8) === 'icmpping' || key.substring(0, 7) === 'vmware.';
+			const object_switcher = globalAllObjForViewSwitcher['type'];
+
+			toggle_fields.forEach((element_id) =>
+				object_switcher[set_hidden ? 'hideObj' : 'showObj']({id: element_id})
+			);
+		}
+		else if (type == <?= ITEM_TYPE_ZABBIX_ACTIVE ?>) {
 			const toggle_fields = [
 				'delay',
 				'js-item-delay-label',
@@ -99,25 +114,42 @@
 				'js-item-flex-intervals-label',
 				'js-item-flex-intervals-field'
 			];
-			const set_hidden = (key.substr(0, 8) === 'mqtt.get'),
-				object_switcher = globalAllObjForViewSwitcher['type'];
+			const set_hidden = key.substring(0, 8) === 'mqtt.get';
+			const object_switcher = globalAllObjForViewSwitcher['type'];
+
+			toggle_fields.forEach((element_id) =>
+				object_switcher[set_hidden ? 'hideObj' : 'showObj']({id: element_id})
+			);
+		}
+		else if (type == <?= ITEM_TYPE_SNMP ?>) {
+			const toggle_fields = [
+				'js-item-timeout-label',
+				'js-item-timeout-field'
+			];
+			const snmp_oid = document.getElementById('snmp_oid').value;
+			const set_hidden = snmp_oid.substring(0, 4) !== 'get[' && snmp_oid.substring(0, 5) !== 'walk[';
+			const object_switcher = globalAllObjForViewSwitcher['type'];
 
 			toggle_fields.forEach((element_id) =>
 				object_switcher[set_hidden ? 'hideObj' : 'showObj']({id: element_id})
 			);
 		}
 
+		item_form.inherited_timeout.value = item_form.inherited_timeouts[type] || '';
+
+		if (item_form.custom_timeout.querySelector(':checked').value == <?= ZBX_ITEM_CUSTOM_TIMEOUT_DISABLED ?>) {
+			item_form.timeout.value = item_form.inherited_timeout.value;
+		}
+
 		$('label[for=interfaceid]').toggleClass('<?= ZBX_STYLE_FIELD_LABEL_ASTERISK ?>', !interface_optional);
 		$('input[name=interfaceid]').prop('aria-required', !interface_optional);
+
+		$('z-select[name="value_type"]').trigger('change');
 	}
 
 	jQuery(document).ready(function($) {
 		$('#delayFlexTable').on('click', 'input[type="radio"]', function() {
-			if ($(this).prop('readonly')) {
-				return;
-			}
-
-			const rowNum = $(this).attr('id').split('_')[2];
+			var rowNum = $(this).attr('id').split('_')[2];
 
 			if ($(this).val() == <?= ITEM_DELAY_FLEXIBLE; ?>) {
 				$('#delay_flex_' + rowNum + '_schedule').hide();
@@ -131,7 +163,7 @@
 			}
 		});
 
-		$('#delayFlexTable').dynamicRows({template: '#delayFlexRow'});
+		$('#delayFlexTable').dynamicRows({template: '#delayFlexRow', allow_empty: true});
 
 		new CViewSwitcher('authtype', 'change', item_form.field_switches.for_authtype);
 
@@ -145,9 +177,9 @@
 			new CViewSwitcher('allow_traps', 'change', item_form.field_switches.for_traps);
 		}
 
-		$("#key").on('keyup change', updateItemFormElements);
+		$("#key, #snmp_oid").on('keyup change', updateItemFormElements);
 
-		$('#parameters_table').dynamicRows({template: '#parameters_table_row'});
+		$('#parameters_table').dynamicRows({template: '#parameters_table_row', allow_empty: true});
 
 		const item_interface_types = item_form.interface_types;
 		const interface_ids_by_types = {};
@@ -160,15 +192,52 @@
 			interface_ids_by_types[interface.type].push(interface.interfaceid);
 		}
 
+		$('z-select[name="value_type"]').change(function() {
+			const ITEM_VALUE_TYPE_BINARY = <?= ITEM_VALUE_TYPE_BINARY?>,
+				binary_selected = this.value == ITEM_VALUE_TYPE_BINARY,
+				disable_binary = $('#type').val() != <?= ITEM_TYPE_DEPENDENT ?>;
+
+			this.getOptionByValue(ITEM_VALUE_TYPE_BINARY).hidden = disable_binary;
+			document.querySelector('z-select[name="value_type_steps"]')
+				.getOptionByValue(ITEM_VALUE_TYPE_BINARY)
+				.hidden = disable_binary;
+
+			if (binary_selected && disable_binary) {
+				this.value = this.getOptions().find((option) => option.value != ITEM_VALUE_TYPE_BINARY).value;
+				$('#type').trigger('change');
+
+				return false;
+			}
+		});
+
 		$('#type')
 			.change(function() {
 				updateItemFormElements();
 				organizeInterfaces(interface_ids_by_types, item_interface_types, parseInt(this.value, 10));
 
 				setAuthTypeLabel();
+
+				if (item_type_lookup.form !== null) {
+					item_type_lookup.update();
+				}
 			})
 			.trigger('change');
 
+		item_form.custom_timeout.addEventListener('change', () => {
+			if (item_form.custom_timeout.querySelector(':checked').value == <?= ZBX_ITEM_CUSTOM_TIMEOUT_ENABLED ?>) {
+				item_form.timeout.disabled = false;
+				item_form.timeout.style.display = '';
+				item_form.inherited_timeout.style.display = 'none';
+			}
+			else {
+				item_form.timeout.disabled = true;
+				item_form.timeout.style.display = 'none';
+				item_form.inherited_timeout.style.display = '';
+			}
+		});
+
+		item_form.custom_timeout.dispatchEvent(new Event('change'));
+
 		$('#test_item').on('click', function() {
 			var step_nums = [];
 			$('z-select[name^="preprocessing"][name$="[type]"]', $('#preprocessing')).each(function() {
@@ -185,23 +254,17 @@
 
 		$('[data-action="parse_url"]').click(function(e) {
 			const url_node = $(this).siblings('[name="url"]');
-			const table = $('#query_fields_pairs').data('editableTable');
+			const table = $('#query-fields-table').data('dynamicRows');
 			const url = parseUrlString(url_node.val());
 
 			if (typeof url === 'object') {
 				if (url.pairs.length > 0) {
 					table.addRows(url.pairs);
-					table.getTableRows()
-						.map(function() {
-							const empty = $(this).find('input[type="text"]').map(function() {
-								return ($(this).val() === '') ? this : null;
-							});
-
-							return (empty.length == 2) ? this : null;
-						})
-						.map(function() {
-							table.removeRow(this);
-						});
+					table.removeRows(row =>
+						[...row.querySelectorAll('[name^="query_fields"]')]
+							.filter(field => field.value === '')
+							.length == 2
+					);
 				}
 
 				url_node.val(url.url);
@@ -264,16 +327,12 @@
 
 			this.updateKeyTypeSuggestions();
 
-			this.preprocessing_tab_type_field.addEventListener('change', (e) => {
-				this.item_tab_type_field.value = this.preprocessing_tab_type_field.value;
-			});
-
 			this.item_tab_type_field.addEventListener('change', (e) => {
 				this.preprocessing_tab_type_field.value = this.item_tab_type_field.value;
 
 				this.updateHintDisplay();
 
-				// 'Do not keep trends' for Calculated with string-types of information is forced on Item save.
+				// 'Do not store' trends for Calculated with string-types of information is forced on Item save.
 				if (this.item_type.value == <?=ITEM_TYPE_CALCULATED ?> && !this.discovered_item) {
 					if (e.target.value == <?= ITEM_VALUE_TYPE_FLOAT ?>
 							|| e.target.value == <?= ITEM_VALUE_TYPE_UINT64 ?>) {
@@ -286,6 +345,11 @@
 				}
 			});
 
+			this.preprocessing_tab_type_field.addEventListener('change', (e) => {
+				this.item_tab_type_field.value = this.preprocessing_tab_type_field.value;
+				this.item_tab_type_field.dispatchEvent(new Event('change'));
+			});
+
 			['change', 'input', 'help_items.paste'].forEach((event_type) => {
 				this.key_field.addEventListener(event_type, (e) => {
 					if (this.preprocessing_active) {
@@ -405,6 +469,12 @@
 			}
 
 			this.item_tab_type_field.dispatchEvent(new CustomEvent('change'));
+		},
+
+		update() {
+			this.inferred_type = null;
+			this.last_lookup = '';
+			this.lookup(this.key_field.value, false);
 		}
 	};
 </script>
